@@ -1,21 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
-
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef belle_sip_dialog_h
 #define belle_sip_dialog_h
@@ -29,17 +29,30 @@ enum belle_sip_dialog_state{
 
 typedef enum belle_sip_dialog_state belle_sip_dialog_state_t;
 
+enum belle_sip_dialog_termination_cause{
+	BELLE_SIP_DIALOG_TERMINATION_CAUSE_NORMAL,
+	BELLE_SIP_DIALOG_TERMINATION_CAUSE_ABORT_NO_ACK
+};
+
+typedef enum belle_sip_dialog_termination_cause belle_sip_dialog_termination_cause_t;
+
 BELLE_SIP_BEGIN_DECLS
 
 BELLESIP_EXPORT const char* belle_sip_dialog_state_to_string(const belle_sip_dialog_state_t state);
 
 BELLESIP_EXPORT belle_sip_request_t *belle_sip_dialog_create_ack(belle_sip_dialog_t *dialog, unsigned int cseq);
 
+BELLESIP_EXPORT bool_t belle_sip_dialog_can_create_asynchronous_request(belle_sip_dialog_t *obj, const char *method);
+
+BELLESIP_EXPORT bool_t belle_sip_dialog_can_create_synchronous_request(belle_sip_dialog_t *obj, const char *method);
 
 /**
  * Create a request part of this dialog.
+ * @param dialog dialog associated to the request
+ * @param method method of the request
 **/
 BELLESIP_EXPORT belle_sip_request_t *belle_sip_dialog_create_request(belle_sip_dialog_t *dialog, const char *method);
+
 /**
  * Create a request within a dialog keeping non system header from an initial request. This function is very useful to resend request after expiration or chalenge.
  * @param obj dialog associated to the request
@@ -51,6 +64,8 @@ BELLESIP_EXPORT belle_sip_request_t * belle_sip_dialog_create_request_from(belle
 /**
  * Create a new request part of this dialog. If dialog is busy (pending transaction), the request can be created anyway and will be sent by the transaction
  * when the dialog becomes available.
+ * @param obj dialog associated to the request
+ * @param method method of the request
 **/
 BELLESIP_EXPORT belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t *obj, const char *method);
 
@@ -79,7 +94,7 @@ BELLESIP_EXPORT const belle_sip_header_address_t *belle_sip_dialog_get_remote_pa
  **/
 BELLESIP_EXPORT unsigned int belle_sip_dialog_get_local_seq_number(const belle_sip_dialog_t *dialog);
 
-unsigned int belle_sip_dialog_get_remote_seq_number(const belle_sip_dialog_t *dialog);
+BELLESIP_EXPORT unsigned int belle_sip_dialog_get_remote_seq_number(const belle_sip_dialog_t *dialog);
 
 BELLESIP_EXPORT const char *belle_sip_dialog_get_local_tag(const belle_sip_dialog_t *dialog);
 
@@ -91,7 +106,7 @@ BELLESIP_EXPORT const belle_sip_list_t* belle_sip_dialog_get_route_set(belle_sip
 
 BELLESIP_EXPORT belle_sip_dialog_state_t belle_sip_dialog_get_state(const belle_sip_dialog_t *dialog);
 /**
- * return the dialog state before last transition. Can be useful to detect early avorted dialogs
+ * return the dialog state before last transition. Can be useful to detect early aborted dialogs
  * @param dialog
  * @returns state
  **/
@@ -114,13 +129,19 @@ BELLESIP_EXPORT belle_sip_transaction_t* belle_sip_dialog_get_last_transaction(c
 
 BELLESIP_EXPORT int belle_sip_dialog_request_pending(const belle_sip_dialog_t *dialog);
 
+BELLESIP_EXPORT int belle_sip_dialog_expired(const belle_sip_dialog_t *dialog);
+
+BELLESIP_EXPORT int belle_sip_dialog_get_request_retry_timeout(const belle_sip_dialog_t *dialog);
+
 /*for debugging purpose only, allow to disable checking for pending transaction*/
 BELLESIP_EXPORT int belle_sip_dialog_pending_trans_checking_enabled( const belle_sip_dialog_t *dialog) ;
 BELLESIP_EXPORT int belle_sip_dialog_enable_pending_trans_checking(belle_sip_dialog_t *dialog, int value) ;
 
-BELLESIP_EXPORT int belle_sip_dialog_expired(const belle_sip_dialog_t *dialog);
+BELLESIP_EXPORT belle_sip_dialog_termination_cause_t belle_sip_dialog_get_termination_cause(const belle_sip_dialog_t *dialog);
+
+/*for testing purposes only*/
+BELLESIP_EXPORT void belle_sip_dialog_set_simulate_lost_ack_enabled(belle_sip_dialog_t *dialog, int enable);
 
 BELLE_SIP_END_DECLS
 
 #endif
-
