@@ -878,6 +878,7 @@ struct belle_sip_dialog{
 	belle_sip_list_t *route_set;
 	belle_sip_header_address_t *remote_target;
 	belle_sip_source_t *expiration_timer;
+	belle_sip_dialog_termination_cause_t termination_cause;
 	char *local_tag;
 	char *remote_tag;
 	unsigned int local_cseq;
@@ -893,6 +894,7 @@ struct belle_sip_dialog{
 	unsigned char is_expired;
 	unsigned char pending_trans_checking_enabled; /*use to disabled pending transaction check at request creation (testing)*/
 	unsigned char is_internal; /*Internal dialogs are those created by refreshers. */
+	unsigned char simulate_lost_ack; /*used by testers*/
 };
 
 belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t);
@@ -1128,20 +1130,10 @@ BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t,belle_sip_body_handler_t)
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
-void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, void *user_data, size_t transfered, size_t expected_total);
 
 /**
  * file manipulation
  */
-/**
- * Parse a directory and return all files in it.
- *
- * @param[in]	path		The directory to be parsed
- * @param[in]	file_type	if not NULL return only the file with the given extension, must include the '.', ex:".pem"
- * @return		a belle_sip list containing all found file or NULL if no file were found or directory doesn't exist. List must be destroyed using belle_sip_list_free_with_data(<ret_list>, belle_sip_free)
- */
-belle_sip_list_t *belle_sip_parse_directory(const char *path, const char *file_type);
-
 typedef struct authorization_context authorization_context_t;
 typedef authorization_context_t belle_sip_authorization_t;
 
