@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle-sip/sip-uri.h"
 #include "belle-sip/parameters.h"
@@ -230,7 +231,7 @@ SIP_URI_GET_SET_INT_PARAM(ttl)
 SIP_URI_HAS_SET_BOOL_PARAM(lr)
 
 
-const belle_sip_parameters_t*	belle_sip_uri_get_headers(const belle_sip_uri_t* uri) {
+const belle_sip_parameters_t* belle_sip_uri_get_headers(const belle_sip_uri_t* uri) {
 	return uri->header_list;
 }
 
@@ -244,7 +245,7 @@ static int uri_strcmp(const char*a,const char*b,int case_sensitive) {
 	int result = 0;
 	size_t index_a=0,index_b=0;
 	char char_a,char_b;
-	
+
 	if (a == NULL && b == NULL) {
 		goto end;
 	}
@@ -270,9 +271,9 @@ end:
 
 #define IS_EQUAL_CASE(a,b) (uri_strcmp(a,b,FALSE)==0)
 #define PARAM_CASE_CMP(uri_a,uri_b,param) \
-		a_param=belle_sip_parameters_get_case_parameter((belle_sip_parameters_t*) uri_a,param); \
-		b_param=belle_sip_parameters_get_case_parameter((belle_sip_parameters_t*) uri_b,param);\
-		if (!IS_EQUAL_CASE(a_param,b_param)) return 0;
+		a_param_value=belle_sip_parameters_get_case_parameter((belle_sip_parameters_t*) uri_a,param); \
+		b_param_value=belle_sip_parameters_get_case_parameter((belle_sip_parameters_t*) uri_b,param);\
+		if (!IS_EQUAL_CASE(a_param_value,b_param_value)) return 0;
 
 /*
  * RFC 3261            SIP: Session Initiation Protocol           June 2002
@@ -284,10 +285,11 @@ end:
    Section 10.3.).  SIP and SIPS URIs are compared for equality
    according to the following rules:
 */
-int belle_sip_uri_equals(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri_b) {
+static int belle_sip_uri_equals_with_omission(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri_b,bool_t uri_omitting) {
 	const belle_sip_list_t *	params;
-	const char* b_param;
-	const char* a_param;
+	belle_sip_param_pair_t * a_param;
+	const char * a_param_value;
+	const char * b_param_value;
 /*
       o  A SIP and SIPS URI are never equivalent.
 */
@@ -302,7 +304,7 @@ int belle_sip_uri_equals(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri
          defined otherwise.
 */
 	if (!IS_EQUAL(uri_a->user,uri_b->user)) return 0;
-	
+
 /*
       o  The ordering of parameters and header fields is not significant
          in comparing SIP and SIPS URIs.
@@ -319,7 +321,27 @@ int belle_sip_uri_equals(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri
 	if (!IS_EQUAL_CASE(uri_a->host,uri_b->host)) {
 		return 0;
 	}
-	if (uri_a->port != uri_b->port) return 0;
+
+	if (uri_omitting) {
+		int port_a = uri_a->port;
+		int port_b = uri_b->port;
+
+		if (port_a == 0) {
+			const char* transport_a = belle_sip_parameters_get_case_parameter((belle_sip_parameters_t*) uri_a, "transport");
+			if (transport_a == NULL) port_a = belle_sip_listening_point_get_well_known_port("UDP");
+			else port_a = belle_sip_listening_point_get_well_known_port(transport_a);
+		}
+		if (port_b == 0) {
+			const char* transport_b = belle_sip_parameters_get_case_parameter((belle_sip_parameters_t*) uri_b, "transport");
+			if (transport_b == NULL) port_b = belle_sip_listening_point_get_well_known_port("UDP");
+			else port_b = belle_sip_listening_point_get_well_known_port(transport_b);
+		}
+
+		if (port_a != port_b) return 0;
+	} else {
+		/* In this case (by default) we respect URI omitting */
+		if (uri_a->port != uri_b->port) return 0;
+	}
 /*
          A URI omitting the user component will not match a URI that
          includes one.  A URI omitting the password component will not
@@ -355,10 +377,10 @@ int belle_sip_uri_equals(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri
 	PARAM_CASE_CMP(uri_a,uri_b,"method")
 	PARAM_CASE_CMP(uri_a,uri_b,"maddr")
 
-
 	for(params=belle_sip_parameters_get_parameters((belle_sip_parameters_t*) uri_a);params!=NULL;params=params->next) {
-		if ((b_param=belle_sip_parameters_get_parameter((belle_sip_parameters_t*) uri_b,(const char*)params->data)) != NULL) {
-			if (!IS_EQUAL_CASE(b_param,(const char*)params->data)) return 0;
+		a_param = (belle_sip_param_pair_t*)(params->data);
+		if ((b_param_value=belle_sip_parameters_get_parameter((belle_sip_parameters_t*) uri_b,a_param->name)) != NULL) {
+			if (!IS_EQUAL_CASE(b_param_value,a_param->value)) return 0;
 		}
 	}
 
@@ -378,6 +400,15 @@ int belle_sip_uri_equals(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri
 	return 1;
 }
 
+int belle_sip_uri_equals(const belle_sip_uri_t* uri_a,const belle_sip_uri_t* uri_b) {
+	return belle_sip_uri_equals_with_omission(uri_a, uri_b, FALSE);
+}
+
+int belle_sip_uri_equals_with_uri_omitting(const belle_sip_uri_t* uri_a, const belle_sip_uri_t* uri_b) {
+	return belle_sip_uri_equals_with_omission(uri_a, uri_b, TRUE);
+}
+
+
 /*uri checker*/
 
 
@@ -442,10 +473,11 @@ typedef struct uri_components {
 } uri_components_t;
 
 
-/*belle sip allows contact header without host because stack will auutomatically put host if missing*/
+/*belle sip allows contact header without host because stack will automatically put host if missing*/
 static  uri_components_t uri_component_use_for_request = 			{"Req.-URI"					,o	,o	,m	,o	,o	,na	,o	,o	,o	,o	,o	,na};
 static  uri_components_t uri_component_use_for_header_to = 			{"Header To"				,o	,o	,m	,na	,o	,na	,na	,na	,na	,na	,o	,na};
 static  uri_components_t uri_component_use_for_header_from = 		{"Header From"				,o	,o	,m	,na	,o	,na	,na	,na	,na	,na	,o	,na};
+static  uri_components_t uri_component_use_for_header_refer_to =	{"Header Refer-To"			,o	,o	,o	,o	,o	,o	,o	,o	,o	,o	,o	,o};
 static  uri_components_t uri_component_use_for_contact_in_reg =		{"Contact in REG"			,o	,o	,/*m*/o	,o	,o	,na	,o	,o	,o	,na	,o	,o};
 static  uri_components_t uri_component_use_for_dialog_ct_rr_ro =	{"Dialog Contact/R-R/Route"	,o	,o	,/*m*/o	,o	,o	,na	,o	,na	,o	,o	,o	,na};
 static  uri_components_t uri_component_use_for_external =			{"External"					,o	,o	,m	,o	,o	,o	,o	,o	,o	,o	,o	,o};
@@ -496,6 +528,8 @@ int belle_sip_uri_check_components_from_context(const belle_sip_uri_t* uri,const
 				|| strcasecmp(BELLE_SIP_RECORD_ROUTE,header_name)==0
 				|| strcasecmp(BELLE_SIP_ROUTE,header_name)==0)
 		return check_uri_components(uri,&uri_component_use_for_dialog_ct_rr_ro);
+	else if (strcasecmp(BELLE_SIP_REFER_TO, header_name) == 0)
+		return check_uri_components(uri, &uri_component_use_for_header_refer_to);
 	else
 		return check_uri_components(uri,&uri_component_use_for_external);
 
