@@ -341,10 +341,32 @@ static int dialog_on_200Ok_timer(belle_sip_dialog_t *dialog){
 static int dialog_on_200Ok_end(belle_sip_dialog_t *dialog){
 	belle_sip_request_t *bye;
 	belle_sip_client_transaction_t *trn;
+	belle_sip_header_reason_t *reason;
+
 	belle_sip_dialog_stop_200Ok_retrans(dialog);
-	belle_sip_error("Dialog [%p] was not ACK'd within T1*64 seconds, it is going to be terminated.",dialog);
+	
+	belle_sip_error("Dialog [%p] was not ACK'd within T1*64 seconds.", dialog);
+	
+	if (dialog->last_transaction){
+		/* Check if a BYE is not already in progress, no need to send a second one.*/
+		const char * request = belle_sip_request_get_method(belle_sip_transaction_get_request(dialog->last_transaction));
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(dialog->last_transaction,belle_sip_client_transaction_t) && strcmp(request, "BYE") == 0){
+			return BELLE_SIP_STOP;
+		}
+	}
+
+	belle_sip_error("Dialog [%p] it is going to be terminated automatically.",dialog);
 	dialog->state=BELLE_SIP_DIALOG_CONFIRMED;
+
 	bye=belle_sip_dialog_create_request(dialog,"BYE");
+	reason=belle_sip_header_reason_new();
+	belle_sip_header_reason_set_protocol(reason,"SIP");
+	belle_sip_header_reason_set_cause(reason,408);
+	belle_sip_header_reason_set_text(reason,"no ACK received");
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(bye), BELLE_SIP_HEADER(reason));
+
+	dialog->termination_cause = BELLE_SIP_DIALOG_TERMINATION_CAUSE_ABORT_NO_ACK;
+
 	trn=belle_sip_provider_create_client_transaction(dialog->provider,bye);
 	BELLE_SIP_TRANSACTION(trn)->is_internal=1; /*don't bother user with this transaction*/
 	belle_sip_client_transaction_send_request(trn);
@@ -535,23 +557,29 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 	int is_retransmition=FALSE;
 	int delete_dialog=FALSE;
 	belle_sip_request_t *req=belle_sip_transaction_get_request(transaction);
+	belle_sip_header_cseq_t* current_cseq=belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(req),belle_sip_header_cseq_t);
+	unsigned int current_transaction_cseq = belle_sip_header_cseq_get_seq_number(current_cseq);
 	belle_sip_response_t *resp=belle_sip_transaction_get_response(transaction);
 	int code=0;
 	int ret = 0;
 	int is_invite = strcmp(belle_sip_request_get_method(req),"INVITE")==0;
 	int is_subscribe = strcmp(belle_sip_request_get_method(req),"SUBSCRIBE")==0;
 	int is_notify = strcmp(belle_sip_request_get_method(req),"NOTIFY")==0;
+	belle_sip_transaction_t *previous_transaction = NULL;
 
 	belle_sip_message("Dialog [%p]: now updated by transaction [%p].",obj, transaction);
 
 	if (resp)
 		code=belle_sip_response_get_status_code(resp);
 
-	if (as_uas && code == 491) { /**/
+	bool_t do_not_update_last_transaction = (as_uas && (code == 491));
+	if (do_not_update_last_transaction) { /**/
 		belle_sip_message("Dialog [%p]: don't update last transaction by transaction [%p].",obj, transaction);
 	} else {
 		belle_sip_object_ref(transaction);
-		if (obj->last_transaction) belle_sip_object_unref(obj->last_transaction);
+		if (obj->last_transaction) {
+			previous_transaction = obj->last_transaction;
+		}
 		obj->last_transaction=transaction;
 	}
 	if (!as_uas){
@@ -559,16 +587,24 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 		SET_OBJECT_PROPERTY(obj,privacy,privacy_header);
 	}
 
-	/*first update local/remote cseq*/
+	/*
+	  first update local/remote cseq
+	  Do not update remote and remote invite CSeq if it is lower than the current one as it may be a retransmission and the next transaction may already be executing
+	*/
 	if (as_uas) {
-		belle_sip_header_cseq_t* cseq=belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(req),belle_sip_header_cseq_t);
-		obj->remote_cseq=belle_sip_header_cseq_get_seq_number(cseq);
-		if (is_invite && code>=200 && code<300)
-			obj->remote_invite_cseq = belle_sip_header_cseq_get_seq_number(cseq);
-		/*else ACK is handled by transaction, not dialog*/
+		bool_t update_remote_invite_cseq = (obj->remote_invite_cseq < current_transaction_cseq);
+		bool_t update_cseq = (obj->remote_cseq < current_transaction_cseq);
+		if ((is_invite && (code>=200) && (code<300) && !update_remote_invite_cseq) || (!update_cseq && !do_not_update_last_transaction)) {
+			obj->last_transaction=previous_transaction;
+		}
+		if (update_cseq) {
+			obj->remote_cseq=current_transaction_cseq;
+		}
+		if (is_invite && (code>=200) && (code<300) && update_remote_invite_cseq) {
+				obj->remote_invite_cseq = current_transaction_cseq;
+		} /*else ACK is handled by transaction, not dialog*/
 	}
 
-
 	switch (obj->state){
 		case BELLE_SIP_DIALOG_NULL:
 			/*always establish a dialog*/
@@ -648,7 +684,9 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 				if (code>=200 && code<300){
 					/*handle possible retransmission of 200Ok */
 					if (!as_uas && (is_retransmition=(belle_sip_dialog_handle_200Ok(obj,resp)==0))) {
-						return is_retransmition;
+						obj->last_transaction=previous_transaction;
+						ret = 1;
+						goto end;
 					} else {
 						if (as_uas)
 							belle_sip_dialog_init_200Ok_retrans(obj,resp);
@@ -677,6 +715,15 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 				if (code>=200 || (code==0 && belle_sip_transaction_get_state(transaction)==BELLE_SIP_TRANSACTION_TERMINATED)){
 					obj->needs_ack=FALSE; /*no longuer need ACK*/
 					if (obj->terminate_on_bye) delete_dialog=TRUE;
+				}else if (!as_uas && code == 0){
+					/* A client BYE transaction is being started */
+					if (previous_transaction && belle_sip_transaction_get_state(previous_transaction) != BELLE_SIP_TRANSACTION_TERMINATED){
+						belle_sip_warning("Forcibly terminating previous transaction as BYE is being sent.");
+						/* Detach the dialog from the transaction, so that the dialog does not get updated for nothing
+						 * by the killed transaction. */
+						belle_sip_transaction_set_dialog(previous_transaction, NULL);
+						belle_sip_transaction_terminate(previous_transaction);
+					}
 				}
 			}else if (is_subscribe){
 				if (belle_sip_dialog_schedule_expiration(obj, (belle_sip_message_t*)req) == BELLE_SIP_STOP
@@ -701,6 +748,13 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 	}
 
 end:
+	if (!do_not_update_last_transaction) {
+		if (obj->last_transaction && (obj->last_transaction == transaction)) {
+			if (previous_transaction) belle_sip_object_unref(previous_transaction);
+		} else {
+			if (transaction) belle_sip_object_unref(transaction);
+		}
+	}
 	if (delete_dialog) belle_sip_dialog_delete(obj);
 	else {
 		belle_sip_dialog_process_queue(obj);
@@ -775,6 +829,7 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 	obj->pending_trans_checking_enabled=1;
 	obj->call_id=(belle_sip_header_call_id_t*)belle_sip_object_ref(call_id);
 	obj->type=type;
+	obj->termination_cause=BELLE_SIP_DIALOG_TERMINATION_CAUSE_NORMAL;
 
 	belle_sip_object_ref(t);
 	obj->last_transaction=t;
@@ -905,15 +960,21 @@ static int dialog_can_create_request(belle_sip_dialog_t *obj, const char *method
 	return TRUE;
 }
 
+bool_t belle_sip_dialog_can_create_asynchronous_request(belle_sip_dialog_t *obj, const char *method){
+	if (!dialog_can_create_request(obj, method)) return FALSE;
+	if (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0){
+		/*we don't allow requests that can update the dialog's state to be sent asynchronously*/
+		belle_sip_error("%s([%p]): [%s] requests are forbidden using this method.",__func__,obj,method);
+		return FALSE;
+	}
+	return TRUE;
+}
+
 belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t *obj, const char *method){
 	belle_sip_request_t *req;
 
-	if (!dialog_can_create_request(obj, method)) return NULL;
-	if (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0){
-		/*we don't allow requests that can update the dialog's state to be sent asynchronously*/
-		belle_sip_error("belle_sip_dialog_create_queued_request([%p]): [%s] requests are forbidden using this method.",obj,method);
-		return NULL;
-	}
+	if (!belle_sip_dialog_can_create_asynchronous_request(obj, method)) return NULL;
+
 	req=create_request(obj,method,FALSE);
 	if (req){
 		req->dialog_queued=TRUE;
@@ -921,21 +982,27 @@ belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t
 	return req;
 }
 
-belle_sip_request_t *belle_sip_dialog_create_request(belle_sip_dialog_t *obj, const char *method){
-	belle_sip_request_t *req;
-
-	if (!dialog_can_create_request(obj, method)) return NULL;
+bool_t belle_sip_dialog_can_create_synchronous_request(belle_sip_dialog_t *obj, const char *method){
+	if (!dialog_can_create_request(obj, method)) return FALSE;
 	/*don't prevent to send a BYE in any case */
-	if (	obj->pending_trans_checking_enabled
+	if (obj->pending_trans_checking_enabled
 			&& strcmp(method,"BYE")!=0
 			&& obj->last_transaction
 			&& belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(obj->last_transaction))){
 
 		if (obj->state != BELLE_SIP_DIALOG_EARLY && strcmp(method,"UPDATE")!=0 && strcmp(method,"NOTIFY")!=0) {
-			belle_sip_error("belle_sip_dialog_create_request(): cannot create [%s] request from dialog [%p] while pending [%s] transaction in state [%s]",method,obj,belle_sip_transaction_get_method(obj->last_transaction), belle_sip_transaction_state_to_string(belle_sip_transaction_get_state(obj->last_transaction)));
-			return NULL;
-		} /*else UPDATE transaction can be send in // */
+			belle_sip_error("%s(): cannot create [%s] request from dialog [%p] while pending [%s] transaction in state [%s]",__func__,method,obj,belle_sip_transaction_get_method(obj->last_transaction), belle_sip_transaction_state_to_string(belle_sip_transaction_get_state(obj->last_transaction)));
+			return FALSE;
+		} /*else UPDATE transaction can be sent in // */
 	}
+	return TRUE;
+}
+
+belle_sip_request_t *belle_sip_dialog_create_request(belle_sip_dialog_t *obj, const char *method){
+	belle_sip_request_t *req;
+
+	if (!belle_sip_dialog_can_create_synchronous_request(obj, method)) return NULL;
+
 	belle_sip_dialog_update_local_cseq(obj,method);
 
 	req=create_request(obj,method,TRUE);
@@ -1126,6 +1193,11 @@ void belle_sip_dialog_check_ack_sent(belle_sip_dialog_t*obj){
 		belle_sip_error("Your listener did not ACK'd the 200Ok for your INVITE request. The dialog will be terminated.");
 		req=belle_sip_dialog_create_request(obj,"BYE");
 		if (req){
+			belle_sip_header_reason_t *reason=belle_sip_header_reason_new();
+			belle_sip_header_reason_set_protocol(reason,"SIP");
+			belle_sip_header_reason_set_cause(reason,500);
+			belle_sip_header_reason_set_text(reason,"Internal Error");
+			belle_sip_message_add_header(BELLE_SIP_MESSAGE(req), BELLE_SIP_HEADER(reason));
 			client_trans=belle_sip_provider_create_client_transaction(obj->provider,req);
 			BELLE_SIP_TRANSACTION(client_trans)->is_internal=TRUE; /*internal transaction, don't bother user with 200ok*/
 			belle_sip_client_transaction_send_request(client_trans);
@@ -1156,6 +1228,10 @@ static int belle_sip_dialog_handle_200Ok(belle_sip_dialog_t *obj, belle_sip_resp
 }
 
 int belle_sip_dialog_handle_ack(belle_sip_dialog_t *obj, belle_sip_request_t *ack){
+	if (obj->simulate_lost_ack) {
+		belle_sip_message("Simulating lost ACK for dialog %p", obj);
+		return -1;
+	}
 	belle_sip_header_cseq_t *cseq=belle_sip_message_get_header_by_type(ack,belle_sip_header_cseq_t);
 	if (obj->needs_ack && belle_sip_header_cseq_get_seq_number(cseq)==obj->remote_invite_cseq){
 		belle_sip_message("Incoming INVITE has ACK, dialog is happy");
@@ -1164,7 +1240,7 @@ int belle_sip_dialog_handle_ack(belle_sip_dialog_t *obj, belle_sip_request_t *ac
 		belle_sip_dialog_process_queue(obj);
 		return 0;
 	}
-	belle_sip_message("Dialog ignoring incoming ACK (surely a retransmission)");
+	belle_sip_message("Dialog ignoring incoming ACK (surely a retransmission) to dialog %p with CSeq %0d", obj, belle_sip_header_cseq_get_seq_number(cseq));
 	return -1;
 }
 
@@ -1176,6 +1252,16 @@ int belle_sip_dialog_request_pending(const belle_sip_dialog_t *dialog){
 	return dialog->needs_ack || (dialog->last_transaction ? belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(dialog->last_transaction)) : FALSE);
 }
 
+int belle_sip_dialog_get_request_retry_timeout(const belle_sip_dialog_t *dialog){
+	/* According to RFC3261 - 14.1 */
+	if (!dialog->is_server){
+		return 2100 + 10 * (((belle_sip_random() % (4000 - 2100))) / 10 );
+	}else{
+		return 10 * ((belle_sip_random() % 2000) / 10 );
+	}
+}
+
+
 /* for notify exception
 As per RFC 3265;
 3.3.4. Dialog creation and termination
@@ -1227,9 +1313,7 @@ void belle_sip_dialog_queue_client_transaction(belle_sip_dialog_t *dialog, belle
 
 static void _belle_sip_dialog_process_queue(belle_sip_dialog_t* dialog){
 	belle_sip_client_transaction_t *tr=NULL;
-
 	if (dialog->state==BELLE_SIP_DIALOG_TERMINATED || belle_sip_dialog_request_pending(dialog)) goto end;
-
 	dialog->queued_ct=belle_sip_list_pop_front(dialog->queued_ct,(void**)&tr);
 	if (tr){
 		belle_sip_message("Dialog [%p]: sending queued request [%p].",dialog,tr);
@@ -1268,3 +1352,11 @@ int belle_sip_dialog_enable_pending_trans_checking(belle_sip_dialog_t *dialog, i
 	dialog->pending_trans_checking_enabled = value;
 	return 0;
 }
+
+belle_sip_dialog_termination_cause_t belle_sip_dialog_get_termination_cause(const belle_sip_dialog_t *dialog) {
+	return dialog->termination_cause;
+}
+
+void belle_sip_dialog_set_simulate_lost_ack_enabled(belle_sip_dialog_t *dialog, int enable) {
+	dialog->simulate_lost_ack = (unsigned char) enable;
+}
