@@ -1,37 +1,38 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
 #include "grammars/belle_sip_messageLexer.h"
 #include "grammars/belle_sip_messageParser.h"
 
 #include "belle_sip_internal.h"
 
-
 typedef struct _headers_container {
 	char* name;
 	belle_sip_list_t* header_list;
 } headers_container_t;
 
-/*reference is 
+/*reference is
  * http://www.iana.org/assignments/sip-parameters/sip-parameters.xhtml#sip-parameters-2
  */
 static const char * expand_name(const char *name){
 	const char *full_name=NULL;
-	
+
 	if (strlen(name)>1) return name;
 	switch(name[0]){
 		case 'a':
@@ -82,7 +83,7 @@ static const char * expand_name(const char *name){
 static headers_container_t* belle_sip_message_headers_container_new(const char* name) {
 	headers_container_t* headers_container = belle_sip_new0(headers_container_t);
 	headers_container->name = belle_sip_strdup(expand_name(name));
-	return  headers_container;
+	return headers_container;
 }
 
 static void belle_sip_headers_container_delete(headers_container_t *obj){
@@ -95,6 +96,8 @@ static void belle_sip_message_destroy(belle_sip_message_t *msg){
 	belle_sip_list_free_with_data(msg->header_list,(void (*)(void*))belle_sip_headers_container_delete);
 	if (msg->body_handler)
 		belle_sip_object_unref(msg->body_handler);
+	if (msg->multipart_body_cache)
+		bctbx_free(msg->multipart_body_cache);
 }
 
 /*very sub-optimal clone method */
@@ -150,7 +153,7 @@ static int belle_sip_headers_container_comp_func(const headers_container_t *a, c
 }
 
 void belle_sip_message_init(belle_sip_message_t *message){
-	
+
 }
 
 headers_container_t* belle_sip_headers_container_get(const belle_sip_message_t* message,const char* header_name) {
@@ -183,12 +186,12 @@ void belle_sip_message_add_header(belle_sip_message_t *message,belle_sip_header_
 void belle_sip_message_add_headers(belle_sip_message_t *message, const belle_sip_list_t *header_list){
 	const char *hname;
 	headers_container_t *headers_container;
-	
+
 	if (header_list == NULL) return;
-	
+
 	hname=belle_sip_header_get_name(BELLE_SIP_HEADER((header_list->data)));
 	headers_container=get_or_create_container(message,hname);
-	
+
 	for(;header_list!=NULL;header_list=header_list->next){
 		belle_sip_header_t *h=BELLE_SIP_HEADER(header_list->data);
 		if (strcmp(belle_sip_header_get_name(h),hname)!=0){
@@ -319,7 +322,7 @@ belle_sip_error_code belle_sip_headers_marshal(belle_sip_message_t *message, cha
 #ifdef BELLE_SIP_WORKAROUND_TECHNICOLOR_SIP_ALG_ROUTER_BUG
 	belle_sip_header_t *content_length=NULL;
 #endif
-	
+
 	for(headers_list=message->header_list;headers_list!=NULL;headers_list=headers_list->next){
 		for(header_list=((headers_container_t*)(headers_list->data))->header_list
 				;header_list!=NULL
@@ -391,7 +394,7 @@ belle_sip_error_code belle_sip_request_marshal(belle_sip_request_t* request, cha
 	if (error!=BELLE_SIP_OK) return error;
 	error=belle_sip_headers_marshal(BELLE_SIP_MESSAGE(request),buff,buff_size,offset);
 	if (error!=BELLE_SIP_OK) return error;
-	
+
 	return error;
 }
 
@@ -476,9 +479,12 @@ belle_sip_body_handler_t *belle_sip_message_get_body_handler(const belle_sip_mes
 }
 
 void belle_sip_message_set_body_handler(belle_sip_message_t *msg, belle_sip_body_handler_t *body_handler){
-	belle_sip_header_content_length_t *content_length_header = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_length_t);
-	belle_sip_header_content_type_t *content_type_header = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
-
+	const belle_sip_header_content_length_t *content_length_header = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_length_t);
+	const belle_sip_header_content_type_t *content_type_header = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
+	const belle_sip_header_t *content_encoding_header = belle_sip_message_get_header(msg, "Content-Encoding");
+	const belle_sip_list_t *body_handler_headers = NULL;
+	if (body_handler)
+		body_handler_headers = belle_sip_body_handler_get_headers(body_handler);
 	/* In case of multipart message, we must add the message Content-Type header containing the boundary */
 	if (body_handler != NULL) {
 		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(body_handler, belle_sip_multipart_body_handler_t)){
@@ -500,9 +506,11 @@ void belle_sip_message_set_body_handler(belle_sip_message_t *msg, belle_sip_body
 						}
 					}
 					if (first_part_content_type) {
-						char *type_slash_subtype = belle_sip_strdup_printf("%s/%s"
-																		   , belle_sip_header_content_type_get_type(first_part_content_type)
-																		   , belle_sip_header_content_type_get_subtype(first_part_content_type));
+						char *type_slash_subtype = belle_sip_strdup_printf(
+							"%s/%s",
+							belle_sip_header_content_type_get_type(first_part_content_type),
+							belle_sip_header_content_type_get_subtype(first_part_content_type)
+						);
 						belle_sip_parameters_set_parameter(BELLE_SIP_PARAMETERS(content_type), "type", type_slash_subtype);
 						belle_sip_free(type_slash_subtype);
 					} else {
@@ -512,22 +520,33 @@ void belle_sip_message_set_body_handler(belle_sip_message_t *msg, belle_sip_body
 					belle_sip_error("Multipart related body handler [%p] cannot be set without first part",body_handler);
 				}
 				belle_sip_header_content_type_set_subtype(content_type, "related");
-				
 			} else {
 				belle_sip_header_content_type_set_subtype(content_type, "form-data");
 			}
-			belle_sip_parameters_set_parameter(BELLE_SIP_PARAMETERS(content_type), "boundary", BELLESIP_MULTIPART_BOUNDARY);
-			belle_sip_message_add_header(BELLE_SIP_MESSAGE(msg), BELLE_SIP_HEADER(content_type));
+
+			while (body_handler_headers) {
+				belle_sip_header_t *header = BELLE_SIP_HEADER(body_handler_headers->data);
+				belle_sip_message_add_header(BELLE_SIP_MESSAGE(msg), header);
+				body_handler_headers = body_handler_headers->next;
+			}
+			const belle_sip_header_content_type_t *new_content_type_header = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
+			if (!new_content_type_header || !belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(new_content_type_header), "boundary")) {
+				belle_sip_parameters_set_parameter(BELLE_SIP_PARAMETERS(content_type), "boundary", belle_sip_multipart_body_handler_get_boundary(multipart_body_handler));
+				belle_sip_message_add_header(BELLE_SIP_MESSAGE(msg), BELLE_SIP_HEADER(content_type));
+			}
 		} else {
 			const belle_sip_list_t *headers = belle_sip_body_handler_get_headers(body_handler);
 			for(; headers != NULL; headers = headers->next) {
 				belle_sip_header_t *header = BELLE_SIP_HEADER(headers->data);
 				if (strcasecmp(belle_sip_header_get_name(header),BELLE_SIP_CONTENT_LENGTH ) == 0 && content_length_header)
 					belle_sip_message_remove_header_from_ptr(msg, BELLE_SIP_HEADER(content_length_header));
-				
+
 				if (strcasecmp(belle_sip_header_get_name(header),BELLE_SIP_CONTENT_TYPE ) == 0 && content_type_header)
 					belle_sip_message_remove_header_from_ptr(msg, BELLE_SIP_HEADER(content_type_header));
-				
+
+				if (strcasecmp(belle_sip_header_get_name(header),"Content-Encoding") == 0 && content_encoding_header)
+					belle_sip_message_remove_header_from_ptr(msg, BELLE_SIP_HEADER(content_encoding_header));
+
 				belle_sip_message_add_header(BELLE_SIP_MESSAGE(msg), header);
 			}
 		}
@@ -539,15 +558,35 @@ void belle_sip_message_set_body_handler(belle_sip_message_t *msg, belle_sip_body
 	SET_OBJECT_PROPERTY(msg,body_handler,body_handler);
 }
 
-const char* belle_sip_message_get_body(belle_sip_message_t *msg) {
-	if (msg->body_handler==NULL) return NULL;
-	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg->body_handler, belle_sip_memory_body_handler_t)){
-		return (const char*)belle_sip_memory_body_handler_get_buffer(
+static const char *get_multipart_body (belle_sip_message_t *msg) {
+	if (!BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg->body_handler, belle_sip_multipart_body_handler_t))
+		return NULL;
+
+	char *old_body = msg->multipart_body_cache;
+	char *body = belle_sip_object_to_string(msg->body_handler);
+	if (old_body) {
+		// Returns the same pointer if no changes.
+		if (!strcmp(body, old_body)) {
+			bctbx_free(body);
+			return old_body;
+		}
+		bctbx_free(msg->multipart_body_cache);
+	}
+	msg->multipart_body_cache = body;
+	return body;
+}
+
+const char *belle_sip_message_get_body(belle_sip_message_t *msg) {
+	if (!msg->body_handler) return NULL;
+	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg->body_handler, belle_sip_memory_body_handler_t)) {
+		return (const char *)belle_sip_memory_body_handler_get_buffer(
 			BELLE_SIP_MEMORY_BODY_HANDLER(msg->body_handler)
 		);
 	}
-	belle_sip_error("belle_sip_message_get_body(): body cannot be returned as pointer.");
-	return NULL;
+	const char *body = get_multipart_body(msg);
+	if (!body)
+		belle_sip_error("belle_sip_message_get_body(): body cannot be returned as pointer.");
+	return body;
 }
 
 size_t belle_sip_message_get_body_size(const belle_sip_message_t *msg){
@@ -579,61 +618,62 @@ typedef struct code_phrase{
 } code_phrase_t;
 
 static code_phrase_t well_known_codes[]={
-	{	100		,		"Trying"	},
-	{	101		,		"Dialog establishment"	},
-	{	180		,		"Ringing"				},
-	{	181		,		"Call is being forwarded"	},
-	{	182		,		"Queued"	},
-	{	183		,		"Session progress"	},
-	{	200		,		"Ok"				},
-	{	202		,		"Accepted"	},
-	{	300		,		"Multiple choices"	},
-	{	301		,		"Moved permanently"	},
-	{	302		,		"Moved temporarily"	},
-	{	305		,		"Use proxy"	},
-	{	380		,		"Alternate contact"	},
-	{	400		,		"Bad request"		},
-	{	401		,		"Unauthorized"		},
-	{	402		,		"Payment required"	},
-	{	403		,		"Forbidden"	},
-	{	404		,		"Not found"	},
-	{	405		,		"Method not allowed"	},
-	{	406		,		"Not acceptable"	},
-	{	407		,		"Proxy authentication required"	},
-	{	408		,		"Request timeout"	},
-	{	410		,		"Gone"	},
-	{	412		,		"Conditional Request Failed" }, /*rfc3903*/
-	{	413		,		"Request entity too large"	},
-	{	414		,		"Request-URI too long"	},
-	{	415		,		"Unsupported media type"	},
-	{	416		,		"Unsupported URI scheme"	},
-	{	420		,		"Bad extension"	},
-	{	421		,		"Extension required"	},
-	{	423		,		"Interval too brief"	},
-	{	480		,		"Temporarily unavailable"	},
-	{	481		,		"Call/transaction does not exist"	},
-	{	482		,		"Loop detected"	},
-	{	483		,		"Too many hops"	},
-	{	484		,		"Address incomplete"	},
-	{	485		,		"Ambiguous"	},
-	{	486		,		"Busy here"	},
-	{	487		,		"Request terminated"	},
-	{	488		,		"Not acceptable here"	},
-	{	489		,		"Bad Event"	}, /*rfc3265*/
-	{	491		,		"Request pending"	},
-	{	493		,		"Undecipherable"	},
-	{	500		,		"Server internal error"	},
-	{	501		,		"Not implemented"	},
-	{	502		,		"Bad gateway"	},
-	{	503		,		"Service unavailable"	},
-	{	504		,		"Server time-out"	},
-	{	505		,		"Version not supported"	},
-	{	513		,		"Message too large"	},
-	{	600		,		"Busy everywhere"	},
-	{	603		,		"Decline"	},
-	{	604		,		"Does not exist anywhere"	},
-	{	606		,		"Not acceptable"	},
-	{	0			,		NULL	}
+	{ 100, "Trying"},
+	{ 101, "Dialog establishment"},
+	{ 180, "Ringing"},
+	{ 181, "Call is being forwarded"},
+	{ 182, "Queued"},
+	{ 183, "Session progress"},
+	{ 200, "Ok"},
+	{ 202, "Accepted"},
+	{ 300, "Multiple choices"},
+	{ 301, "Moved permanently"},
+	{ 302, "Moved temporarily"},
+	{ 305, "Use proxy"},
+	{ 380, "Alternate contact"},
+	{ 400, "Bad request"},
+	{ 401, "Unauthorized"},
+	{ 402, "Payment required"},
+	{ 403, "Forbidden"},
+	{ 404, "Not found"},
+	{ 405, "Method not allowed"},
+	{ 406, "Not acceptable"},
+	{ 407, "Proxy authentication required"},
+	{ 408, "Request timeout"},
+	{ 410, "Gone"},
+	{ 412, "Conditional Request Failed" }, /*rfc3903*/
+	{ 413, "Request entity too large"},
+	{ 414, "Request-URI too long"},
+	{ 415, "Unsupported media type"},
+	{ 416, "Unsupported URI scheme"},
+	{ 420, "Bad extension"},
+	{ 421, "Extension required"},
+	{ 422, "Session Interval Too Small"}, /*rfc4028*/
+	{ 423, "Interval too brief"},
+	{ 480, "Temporarily unavailable"},
+	{ 481, "Call/transaction does not exist"},
+	{ 482, "Loop detected"},
+	{ 483, "Too many hops"},
+	{ 484, "Address incomplete"},
+	{ 485, "Ambiguous"},
+	{ 486, "Busy here"},
+	{ 487, "Request terminated"},
+	{ 488, "Not acceptable here"},
+	{ 489, "Bad Event"}, /*rfc3265*/
+	{ 491, "Request pending"},
+	{ 493, "Undecipherable"},
+	{ 500, "Server internal error"},
+	{ 501, "Not implemented"},
+	{ 502, "Bad gateway"},
+	{ 503, "Service unavailable"},
+	{ 504, "Server time-out"},
+	{ 505, "Version not supported"},
+	{ 513, "Message too large"},
+	{ 600, "Busy everywhere"},
+	{ 603, "Decline"},
+	{ 604, "Does not exist anywhere"},
+	{ 606, "Not acceptable"},
+	{ 0, NULL	}
 };
 
 const char *belle_sip_get_well_known_reason_phrase(int status_code){
@@ -665,7 +705,7 @@ belle_sip_error_code belle_sip_response_marshal(belle_sip_response_t *resp, char
 						,"SIP/2.0 %i %s\r\n"
 						,belle_sip_response_get_status_code(resp)
 						,belle_sip_response_get_reason_phrase(resp)?belle_sip_response_get_reason_phrase(resp):"");
-	
+
 	if (error!=BELLE_SIP_OK) return error;
 	error=belle_sip_headers_marshal(BELLE_SIP_MESSAGE(resp),buff,buff_size,offset);
 	if (error!=BELLE_SIP_OK) return error;
@@ -697,14 +737,16 @@ static int is_authorized_uri_header(const char* header_name) {
 
 }
 
-belle_sip_request_t* belle_sip_request_create(belle_sip_uri_t *requri, const char* method,
-                                         belle_sip_header_call_id_t *callid,
-                                         belle_sip_header_cseq_t * cseq,
-                                         belle_sip_header_from_t *from,
-                                         belle_sip_header_to_t *to,
-                                         belle_sip_header_via_t *via,
-                                         int max_forward)
-{
+belle_sip_request_t* belle_sip_request_create (
+	belle_sip_uri_t *requri,
+	const char* method,
+	belle_sip_header_call_id_t *callid,
+	belle_sip_header_cseq_t * cseq,
+	belle_sip_header_from_t *from,
+	belle_sip_header_to_t *to,
+	belle_sip_header_via_t *via,
+	int max_forward
+) {
 	belle_sip_request_t *ret=belle_sip_request_new();
 	belle_sip_header_max_forwards_t *mf=belle_sip_header_max_forwards_new();
 	belle_sip_list_t* iterator;
@@ -796,7 +838,7 @@ belle_sip_response_t *belle_sip_response_create_from_request(belle_sip_request_t
 	belle_sip_header_t *h;
 	belle_sip_header_to_t *to;
 	const belle_sip_list_t *vias;
-	
+
 	belle_sip_response_init_default(resp,status_code,NULL);
 	if (status_code==100 && (h=belle_sip_message_get_header((belle_sip_message_t*)req,"timestamp"))){
 		belle_sip_message_add_header((belle_sip_message_t*)resp,h);
@@ -845,7 +887,7 @@ void belle_sip_response_fill_for_dialog(belle_sip_response_t *obj, belle_sip_req
 			/*add a dummy contact to be filled by channel later*/
 			belle_sip_message_add_header((belle_sip_message_t*)obj,(belle_sip_header_t*)belle_sip_header_contact_new());
 		}
-	}	
+	}
 }
 
 belle_sip_hop_t* belle_sip_response_get_return_hop(belle_sip_response_t *msg){
@@ -853,7 +895,7 @@ belle_sip_hop_t* belle_sip_response_get_return_hop(belle_sip_response_t *msg){
 	if (via){
 		const char *host=belle_sip_header_via_get_received(via) ? belle_sip_header_via_get_received(via) : belle_sip_header_via_get_host(via);
 		int port=belle_sip_header_via_get_rport(via)>0 ? belle_sip_header_via_get_rport(via) : belle_sip_header_via_get_listening_port(via);
-		return belle_sip_hop_new(belle_sip_header_via_get_transport_lowercase(via),NULL,host,port);
+		return belle_sip_hop_new(belle_sip_header_via_get_transport_lowercase(via), belle_sip_header_via_get_host(via), host, port);
 	}
 	return NULL;
 }
@@ -1007,12 +1049,31 @@ static message_header_list_t mandatory_headers[] = {
 	return 0;
 }
 */
+
+/* This mainly check for mandatory parameters in some headers. */
+static int belle_sip_message_check_headers_integrity(const belle_sip_message_t *message){
+	{
+		belle_sip_header_from_t *from = belle_sip_message_get_header_by_type(message, belle_sip_header_from_t);
+		const char *from_tag;
+		if (!from) return 0;
+		from_tag = belle_sip_header_from_get_tag(from);
+		if (!from_tag || strlen(from_tag) == 0) {
+			belle_sip_error("No tag in from header.");
+			return 0;
+		}
+	}
+	/* Note: a via without branch parameter is allowed for backward compatibility with RFC2543. */
+	return 1;
+}
+
+/* This checks the presence for mandatory headers, per method, according to RFC3261. 
+ * Returns 1 if the message is compliant, 0 otherwise. */
 int belle_sip_message_check_headers(const belle_sip_message_t* message) {
 	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(message,belle_sip_request_t)) {
 		int i;
 		belle_sip_header_via_t *via;
 		const char * method = belle_sip_request_get_method(BELLE_SIP_REQUEST(message));
-		
+
 		for (i=0;mandatory_headers[i].method!=NULL;i++) {
 			if ( (strcasecmp(method,mandatory_headers[i].method)==0) ||
 				 (mandatory_headers[i].method[0] == '*') ){
@@ -1023,15 +1084,12 @@ int belle_sip_message_check_headers(const belle_sip_message_t* message) {
 						return 0;
 					}
 				}
-				return 1;
 			}
 		}
-		via=belle_sip_message_get_header_by_type(message,belle_sip_header_via_t);
-		if (!via || belle_sip_header_via_get_branch(via)==NULL) return 0;
+		
 	}
-	/*else fixme should also check responses*/
-	return 1;
-
+	/*else FIXME should also check in esponses*/
+	return belle_sip_message_check_headers_integrity(message);
 }
 
 int belle_sip_request_check_uris_components(const belle_sip_request_t* request) {
