@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2012  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <bctoolbox/defs.h>
 
@@ -41,21 +42,24 @@ struct belle_sip_refresher {
 	belle_sip_refresher_listener_t listener;
 	belle_sip_source_t* timer;
 	belle_sip_client_transaction_t* transaction;
-	belle_sip_request_t* first_acknoleged_request; /*store first request sucessfully acknoleged, usefull to re-build a dialog iff needed*/
+	belle_sip_request_t* first_acknowledged_request; /*store first request sucessfully acknowledged, usefull to re-build a dialog if needed*/
 	belle_sip_dialog_t* dialog; /*Cannot rely on transaction to store dialog because of belle_sip_transaction_reset_dialog*/
 	char* realm;
 	int target_expires;
+	int session_expires;
 	int obtained_expires;
 	belle_sip_refresher_state_t state;
 	void* user_data;
 	int retry_after;
 	belle_sip_list_t* auth_events;
 	int auth_failures;
-	int on_io_error; /*flag to avoid multiple error notification*/
 	int number_of_retry; /*counter to count number of unsuccesfull retry, used to know when to retry*/
 	timer_purpose_t timer_purpose;
 	unsigned char manual;
 	unsigned int publish_pending;
+	unsigned char on_io_error; /*flag to avoid multiple error notification*/
+	unsigned char contact_fixing; /* flag to indicate that we are submiting an updated REGISTER to fix contact after
+			learning IP/port from previous response*/
 };
 static void set_or_update_dialog(belle_sip_refresher_t* refresher, belle_sip_dialog_t* dialog);
 static int set_expires_from_trans(belle_sip_refresher_t* refresher);
@@ -87,9 +91,9 @@ static void retry_later(belle_sip_refresher_t* refresher) {
 }
 
 static void retry_later_on_io_error(belle_sip_refresher_t* refresher) {
-	/*if first retry, sent it in 500 ms*/
+	/*if first retry, sent it immediately. */
 	if (refresher->number_of_retry < 1) {
-		schedule_timer_at(refresher,DEFAULT_INITIAL_RETRY_AFTER_ON_IO_ERROR,RETRY);
+		schedule_timer_at(refresher,0,RETRY);
 		refresher->number_of_retry++;
 	} else {
 		retry_later(refresher);
@@ -115,7 +119,6 @@ static void process_dialog_terminated(belle_sip_listener_t *user_ctx, const bell
 		if (refresher->listener) refresher->listener(refresher,refresher->user_data,481, "dialod terminated", TRUE);
 
 	}
-
 }
 
 static void process_io_error(belle_sip_listener_t *user_ctx, const belle_sip_io_error_event_t *event){
@@ -145,21 +148,24 @@ static void process_io_error(belle_sip_listener_t *user_ctx, const belle_sip_io_
 		/*something went wrong on this provider, checking if my channel is still up*/
 		if (refresher->state==started  /*refresher started or trying to refresh */
 				&& belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(refresher->transaction)) == BELLE_SIP_TRANSACTION_TERMINATED /*else we are notified by transaction error*/
-				&& refresher->transaction->base.channel /*transaction may not have any channel*/
-				&&	(belle_sip_channel_get_state(refresher->transaction->base.channel) == BELLE_SIP_CHANNEL_DISCONNECTED
-								||belle_sip_channel_get_state(refresher->transaction->base.channel) == BELLE_SIP_CHANNEL_ERROR)) {
-			belle_sip_message("refresher [%p] has channel [%p] in state [%s], reporting error"
+				&& refresher->transaction->base.channel /*transaction may not have any channel*/){
+			switch (belle_sip_channel_get_state(refresher->transaction->base.channel)){
+				case BELLE_SIP_CHANNEL_DISCONNECTED:
+				case BELLE_SIP_CHANNEL_ERROR:
+				case BELLE_SIP_CHANNEL_RETRY:
+					belle_sip_message("refresher [%p] has channel [%p] in state [%s], reporting error"
 								,refresher
 								,refresher->transaction->base.channel
 								,belle_sip_channel_state_to_string(belle_sip_channel_get_state(refresher->transaction->base.channel)));
-			if (refresher->state==started) retry_later_on_io_error(refresher);
-			if (refresher->listener) refresher->listener(refresher,refresher->user_data,503, "io error", refresher->state == started);
-			refresher->on_io_error=1;
+					if (refresher->state==started) retry_later_on_io_error(refresher);
+					if (refresher->listener) refresher->listener(refresher,refresher->user_data,503, "io error", refresher->state == started);
+					refresher->on_io_error=1;
+				break;
+				default:
+				break;
+			}
 		}
 		return;
-	}else {
-		/*belle_sip_error("Refresher process_io_error not implemented yet for non transaction/provider source");*/
-		/*nop, because already handle at transaction layer*/
 	}
 }
 
@@ -213,13 +219,17 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 	int response_code = belle_sip_response_get_status_code(response);
 	belle_sip_refresher_t* refresher=(belle_sip_refresher_t*)user_ctx;
 	belle_sip_header_contact_t *contact;
+	belle_sip_header_retry_after_t *retry_after_header;
 	int will_retry = TRUE; /*most error codes are retryable*/
+	retry_after_header = belle_sip_message_get_header_by_type(response,belle_sip_header_retry_after_t);
+	int retry_after_time = retry_after_header ? belle_sip_header_retry_after_get_retry_after(retry_after_header) : DEFAULT_RETRY_AFTER;
 
 
 	if (refresher && (client_transaction !=refresher->transaction))
 		return; /*not for me*/
 
 	set_or_update_dialog(refresher,belle_sip_response_event_get_dialog(event));
+	contact=belle_sip_message_get_header_by_type(response,belle_sip_header_contact_t);
 	/*success case:*/
 	if (response_code>=200 && response_code<300){
 		refresher->auth_failures=0;
@@ -258,14 +268,22 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 		}
 
 		if (refresher->state==started) {
-			if (!refresher->first_acknoleged_request)
-				belle_sip_object_ref(refresher->first_acknoleged_request = request);
-			if (is_contact_address_acurate(refresher,request) || !belle_sip_provider_nat_helper_enabled(client_transaction->base.provider)) {
+			if (!refresher->first_acknowledged_request)
+				belle_sip_object_ref(refresher->first_acknowledged_request = request);
+			if (is_contact_address_acurate(refresher,request) 
+				|| (!belle_sip_provider_nat_helper_enabled(client_transaction->base.provider) || (contact && belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(contact), "pub-gruu"))) ) { /*Disable nat helper in gruu case. Might not be the best fix, maybe better to make reflesh is not mandatory*/
 				schedule_timer(refresher); /*re-arm timer*/
 			} else {
-				belle_sip_message("belle_sip_refresher_start(): refresher [%p] is resubmitting request because contact sent was not correct in original request.",refresher);
-				belle_sip_refresher_refresh(refresher,refresher->target_expires);
-				return;
+				if (!refresher->contact_fixing){
+					belle_sip_message("belle_sip_refresher_start(): refresher [%p] is resubmitting request because contact sent was not correct in original request.",refresher);
+					belle_sip_refresher_refresh(refresher,refresher->target_expires);
+					refresher->contact_fixing = TRUE;
+					return;
+				}else{
+					belle_sip_message("refresher [%p]: unable to fix contact for NAT - asymmetric flow ?", refresher);
+					refresher->contact_fixing = FALSE; /* reset flag and re-arm timer for next refresh */
+					schedule_timer(refresher); /*re-arm timer*/
+				}
 			}
 		}
 		else belle_sip_message("Refresher [%p] not scheduling next refresh, because it was stopped",refresher);
@@ -273,7 +291,7 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 		switch (response_code) {
 		case 301:
 		case 302:
-			contact=belle_sip_message_get_header_by_type(response,belle_sip_header_contact_t);
+
 			if (contact){
 				belle_sip_uri_t *uri=belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(contact));
 				if (uri && belle_sip_refresher_refresh_internal(refresher,refresher->target_expires,TRUE,&refresher->auth_events,uri)==0)
@@ -339,6 +357,15 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 			}
 		}
 		BCTBX_NO_BREAK; /*intentionally no break*/
+		case 404:
+		case 480:
+		case 500: {
+			if (refresher->target_expires>0 && retry_after_header) {
+				schedule_timer_at(refresher, retry_after_time * 1000, RETRY);
+				return; /*do not notify this kind of error*/
+			}
+		}
+		BCTBX_NO_BREAK; /*intentionally no break*/
 		case 505:
 		case 501:
 			/*irrecoverable errors, probably no need to retry later*/
@@ -348,7 +375,9 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 		case 503:
 			if (refresher->target_expires>0) {
 				if (refresher->dialog) retry_later_on_io_error(refresher);
-				else retry_later(refresher);
+				else {
+					schedule_timer_at(refresher, retry_after_time * 1000, RETRY);
+				}
 			}else will_retry = FALSE;
 			break;
 		default:
@@ -405,10 +434,12 @@ static void destroy(belle_sip_refresher_t *refresher){
 	refresher->transaction=NULL;
 	if (refresher->realm) belle_sip_free(refresher->realm);
 	if (refresher->auth_events) refresher->auth_events=belle_sip_list_free_with_data(refresher->auth_events,(void (*)(void*))belle_sip_auth_event_destroy);
-	if (refresher->first_acknoleged_request) belle_sip_object_unref(refresher->first_acknoleged_request);
+	if (refresher->first_acknowledged_request) belle_sip_object_unref(refresher->first_acknowledged_request);
 	if (refresher->dialog) belle_sip_object_unref(refresher->dialog);
 }
 
+
+
 BELLE_SIP_IMPLEMENT_INTERFACE_BEGIN(belle_sip_refresher_t,belle_sip_listener_t)
 	process_dialog_terminated,
 	process_io_error,
@@ -520,16 +551,15 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 						belle_sip_message_set_body(BELLE_SIP_MESSAGE(request), NULL, 0);
 						belle_sip_message_remove_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_CONTENT_TYPE);
 						belle_sip_message_remove_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_CONTENT_LENGTH);
-
 					}
 				}
 				belle_sip_provider_add_authorization(prov,request,old_response,NULL,auth_infos,refresher->realm);
 				break;
 			}
 			case BELLE_SIP_DIALOG_TERMINATED: {
-				if (refresher->first_acknoleged_request) {
+				if (refresher->first_acknowledged_request) {
 					belle_sip_message("Dialog [%p] is in state terminated, recreating a new one for refresher [%p]",dialog,refresher);
-					request = refresher->first_acknoleged_request;
+					request = refresher->first_acknowledged_request;
 					belle_sip_header_cseq_set_seq_number(belle_sip_message_get_header_by_type(request,belle_sip_header_cseq_t)
 														 ,20);
 					belle_sip_parameters_remove_parameter(BELLE_SIP_PARAMETERS(belle_sip_message_get_header_by_type(request,belle_sip_header_to_t)),"tag");
@@ -577,9 +607,9 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 	client_transaction = belle_sip_provider_create_client_transaction(prov,request);
 	client_transaction->base.is_internal=1;
 
-	if (request ==  refresher->first_acknoleged_request) { /*request is now ref by transaction so no need to keepo it*/
-		belle_sip_object_unref(refresher->first_acknoleged_request);
-		refresher->first_acknoleged_request = NULL;
+	if (request ==  refresher->first_acknowledged_request) { /*request is now ref by transaction so no need to keepo it*/
+		belle_sip_object_unref(refresher->first_acknowledged_request);
+		refresher->first_acknowledged_request = NULL;
 	}
 
 	switch (belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(refresher->transaction))) {
@@ -597,6 +627,9 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 	refresher->transaction=client_transaction;
 	belle_sip_object_ref(refresher->transaction);
 
+	/*Dialog may have changed, specially if terminated (I.E timeout). In that case, it will only be autiore-created when response is received from peer, so in the mean time, we reset it*/
+	set_or_update_dialog(refresher, belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(client_transaction)));
+
 	if (belle_sip_client_transaction_send_request_to(client_transaction,requri?requri:preset_route)) { /*send imediatly to requri in case of redirect*/
 		belle_sip_error("Cannot send refresh method [%s] for refresher [%p]"
 				,belle_sip_request_get_method(request)
@@ -644,13 +677,13 @@ belle_sip_header_contact_t* belle_sip_refresher_get_contact(const belle_sip_refr
 
 	if (contact_header_list) {
 		contact_header_list = belle_sip_list_find_custom((belle_sip_list_t*)contact_header_list
-					,(belle_sip_compare_func)belle_sip_header_contact_not_equals
+					,(belle_sip_compare_func)belle_sip_header_contact_not_equals_with_uri_omitting
 					, (const void*)fixed_local_contact);
 		if (!contact_header_list) {
 			/*reset header list*/
 			contact_header_list = belle_sip_message_get_headers(BELLE_SIP_MESSAGE(response),BELLE_SIP_CONTACT);
 			contact_header_list = belle_sip_list_find_custom((belle_sip_list_t*)contact_header_list
-							,(belle_sip_compare_func)belle_sip_header_contact_not_equals
+							,(belle_sip_compare_func)belle_sip_header_contact_not_equals_with_uri_omitting
 							,unfixed_local_contact);
 		}
 		if (!contact_header_list) {
@@ -785,6 +818,11 @@ belle_sip_refresher_t* belle_sip_refresher_new(belle_sip_client_transaction_t* t
 
 	if (belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(transaction))) {
 		set_or_update_dialog(refresher, belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(transaction)));
+		/*if dialog is already in state confirmed, store initial request to be able to re-create dialog in case of expired when trying to refresh again*/
+		if (belle_sip_dialog_get_state(belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(transaction))) == BELLE_SIP_DIALOG_CONFIRMED) {
+			if (!refresher->first_acknowledged_request)
+				belle_sip_object_ref(refresher->first_acknowledged_request = request);
+		}
 	}
 	belle_sip_provider_add_internal_sip_listener(transaction->base.provider,BELLE_SIP_LISTENER(refresher), is_register);
 	if (set_expires_from_trans(refresher)==-1){
@@ -794,9 +832,10 @@ belle_sip_refresher_t* belle_sip_refresher_new(belle_sip_client_transaction_t* t
 		belle_sip_message("Refresher [%p] takes ownership of transaction [%p]",refresher,transaction);
 		transaction->base.is_internal=1;
 		refresher->state=started;
-	}else{
+	} else {
 		belle_sip_refresher_start(refresher);
 	}
+
 	return refresher;
 }
 
