@@ -1,24 +1,45 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 #include "listeningpoint_internal.h"
 
+static int belle_sip_well_known_port = 5060;
+static int belle_sip_well_known_port_tls = 5061;
+
+int belle_sip_stack_get_well_known_port(){
+	return belle_sip_well_known_port;
+}
+
+void belle_sip_stack_set_well_known_port(int value){
+	bctbx_message("belle_sip_stack_set_well_know_port() : set to [%i]", value);
+	belle_sip_well_known_port = value;
+}
+
+int belle_sip_stack_get_well_known_port_tls(){
+	return belle_sip_well_known_port_tls;
+}
+
+void belle_sip_stack_set_well_known_port_tls(int value){
+	bctbx_message("belle_sip_stack_set_well_know_port_tls() : set to [%i]", value);
+	belle_sip_well_known_port_tls = value;
+}
 
 belle_sip_hop_t* belle_sip_hop_new(const char* transport, const char *cname, const char* host,int port) {
 	belle_sip_hop_t* hop = belle_sip_object_new(belle_sip_hop_t);
@@ -40,17 +61,21 @@ belle_sip_hop_t* belle_sip_hop_new_from_uri(const belle_sip_uri_t *uri){
 	const char *host;
 	const char *cname=NULL;
 	const char * transport=belle_sip_uri_get_transport_param(uri);
+	belle_sip_hop_t *hop;
 	if (!transport) {
 		transport=belle_sip_uri_is_secure(uri)?"tls":"udp";
 	}
-	host=belle_sip_uri_get_maddr_param(uri);
+	host = belle_sip_uri_get_maddr_param(uri);
 	if (!host) host=belle_sip_uri_get_host(uri);
-	else cname=belle_sip_uri_get_host(uri);
+	
+	cname=belle_sip_uri_get_host(uri);
 
-	return belle_sip_hop_new(	transport,
+	hop = belle_sip_hop_new(	transport,
 								cname,
 								host,
 								belle_sip_uri_get_listening_port(uri));
+	hop->port_is_explicit = (belle_sip_uri_get_port(uri) > 0);
+	return hop;
 }
 
 belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *uri){
@@ -59,8 +84,14 @@ belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *u
 	const char *scheme=belle_generic_uri_get_scheme(uri);
 	int port=belle_generic_uri_get_port(uri);
 	int well_known_port=0;
-	
-	host=belle_generic_uri_get_host(uri);
+	const char *cname=NULL;
+	host = belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(uri),"maddr");
+
+	if (!host)
+		host = belle_generic_uri_get_host(uri);
+	else
+		cname=belle_generic_uri_get_host(uri);
+
 	if (strcasecmp(scheme,"http")==0) {
 		transport="TCP";
 		well_known_port=80;
@@ -70,7 +101,7 @@ belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *u
 	}
 
 	return belle_sip_hop_new(transport,
-				host,
+				cname,
 				host,
 				port > 0 ? port : well_known_port);
 }
@@ -97,22 +128,58 @@ static void belle_sip_hop_clone(belle_sip_hop_t *hop, const belle_sip_hop_t *ori
 		hop->cname=belle_sip_strdup(orig->cname);
 	if (orig->transport)
 		hop->transport=belle_sip_strdup(orig->transport);
-	
+
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_hop_t);
 BELLE_SIP_INSTANCIATE_VPTR(belle_sip_hop_t,belle_sip_object_t,belle_sip_hop_destroy,belle_sip_hop_clone,NULL,TRUE);
 
+
+struct belle_sip_digest_authentication_policy{
+	belle_sip_object_t base;
+	unsigned char allow_md5;
+	unsigned char allow_no_qop;
+};
+
+static void belle_sip_digest_authentication_policy_clone(belle_sip_digest_authentication_policy_t *obj, const belle_sip_digest_authentication_policy_t *from){
+	obj->allow_md5 = from->allow_md5;
+	obj->allow_no_qop = from->allow_no_qop;
+}
+
+
+GET_SET_INT(belle_sip_digest_authentication_policy, allow_md5, unsigned char)
+
+GET_SET_INT(belle_sip_digest_authentication_policy, allow_no_qop, unsigned char)
+
+
+BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_digest_authentication_policy_t);
+BELLE_SIP_INSTANCIATE_VPTR(belle_sip_digest_authentication_policy_t,belle_sip_object_t,NULL,belle_sip_digest_authentication_policy_clone,NULL,TRUE);
+
+
+belle_sip_digest_authentication_policy_t * belle_sip_digest_authentication_policy_new(void){
+	belle_sip_digest_authentication_policy_t *obj = belle_sip_object_new(belle_sip_digest_authentication_policy_t);
+	obj->allow_md5 = TRUE;
+	obj->allow_no_qop = TRUE;
+	return obj;
+}
+
 static void belle_sip_stack_destroy(belle_sip_stack_t *stack){
 	belle_sip_message("stack [%p] destroyed.",stack);
 	if (stack->dns_user_hosts_file) belle_sip_free(stack->dns_user_hosts_file);
 	if (stack->dns_resolv_conf) belle_sip_free(stack->dns_resolv_conf);
 	belle_sip_object_unref(stack->ml);
+	belle_sip_object_unref(stack->digest_auth_policy);
 	if (stack->http_proxy_host) belle_sip_free(stack->http_proxy_host);
 	if (stack->http_proxy_passwd) belle_sip_free(stack->http_proxy_passwd);
 	if (stack->http_proxy_username) belle_sip_free(stack->http_proxy_username);
 	belle_sip_list_free_with_data(stack->dns_servers, belle_sip_free);
-
+#ifdef HAVE_DNS_SERVICE
+	if (stack->dns_service_queue) {
+		dispatch_release(stack->dns_service_queue);
+		stack->dns_service_queue = NULL;
+	}
+#endif /* HAVE_DNS_SERVICE */
+	bctbx_uninit_logger();
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_stack_t);
@@ -120,6 +187,7 @@ BELLE_SIP_INSTANCIATE_VPTR(belle_sip_stack_t,belle_sip_object_t,belle_sip_stack_
 
 belle_sip_stack_t * belle_sip_stack_new(const char *properties){
 	belle_sip_stack_t *stack=belle_sip_object_new(belle_sip_stack_t);
+	bctbx_init_logger(FALSE);
 	stack->ml=belle_sip_main_loop_new ();
 	stack->timer_config.T1=500;
 	stack->timer_config.T2=4000;
@@ -129,6 +197,12 @@ belle_sip_stack_t * belle_sip_stack_new(const char *properties){
 	stack->dns_srv_enabled=TRUE;
 	stack->dns_search_enabled=TRUE;
 	stack->inactive_transport_timeout=3600; /*one hour*/
+	stack->unreliable_transport_timeout = 120;
+#ifdef HAVE_DNS_SERVICE
+	stack->dns_service_queue = dispatch_queue_create("com.belledonne_communications.belle_sip.dns_service_queue", DISPATCH_QUEUE_SERIAL);
+	stack->use_dns_service = TRUE;
+#endif /* HAVE_DNS_SERVICE */
+	belle_sip_stack_set_digest_authentication_policy(stack, belle_sip_digest_authentication_policy_new());
 	return stack;
 }
 
@@ -161,6 +235,30 @@ void belle_sip_stack_set_dns_timeout(belle_sip_stack_t *stack, int timeout) {
 	stack->dns_timeout = timeout;
 }
 
+unsigned char belle_sip_stack_get_dns_engine(const belle_sip_stack_t *stack) {
+#ifdef HAVE_DNS_SERVICE
+	return stack->use_dns_service==TRUE?BELLE_SIP_DNS_APPLE_DNS_SERVICE:BELLE_SIP_DNS_DNS_C;
+#else /* HAVE_DNS_SERVICE */
+	return BELLE_SIP_DNS_DNS_C;
+#endif /* else HAVE_DNS_SERVICE*/
+}
+
+void belle_sip_stack_set_dns_engine(belle_sip_stack_t *stack, unsigned char dns_engine) {
+#ifdef HAVE_DNS_SERVICE
+	switch (dns_engine) {
+		case BELLE_SIP_DNS_APPLE_DNS_SERVICE:
+			stack->use_dns_service = TRUE;
+			break;
+		case BELLE_SIP_DNS_DNS_C:
+			stack->use_dns_service = FALSE;
+			break;
+		default:
+			belle_sip_error("Cannot set DNS engine to unknown value : %d", dns_engine);
+	}
+#endif /* HAVE_DNS_SERVICE */
+}
+
+
 unsigned char belle_sip_stack_dns_srv_enabled(const belle_sip_stack_t *stack) {
 	return stack->dns_srv_enabled;
 }
@@ -201,7 +299,11 @@ belle_sip_provider_t *belle_sip_stack_create_provider(belle_sip_stack_t *s, bell
 }
 
 belle_http_provider_t *belle_sip_stack_create_http_provider(belle_sip_stack_t *s, const char *bind_ip){
-	belle_http_provider_t *p=belle_http_provider_new(s, bind_ip);
+	return belle_sip_stack_create_http_provider_with_transports(s, bind_ip, BELLE_SIP_HTTP_TRANSPORT_TCP|BELLE_SIP_HTTP_TRANSPORT_TLS);
+}
+
+belle_http_provider_t *belle_sip_stack_create_http_provider_with_transports(belle_sip_stack_t *s, const char *bind_ip, const uint8_t transports){
+	belle_http_provider_t *p=belle_http_provider_new(s, bind_ip, transports);
 	return p;
 }
 
@@ -250,6 +352,10 @@ void belle_sip_stack_set_resolver_send_error(belle_sip_stack_t *stack, int send_
 	stack->resolver_send_error = send_error;
 }
 
+void belle_sip_stack_simulate_non_working_srv(belle_sip_stack_t *stack, int yesno){
+	stack->simulate_non_working_srv = yesno;
+}
+
 const char * belle_sip_stack_get_dns_user_hosts_file(const belle_sip_stack_t *stack) {
 	return stack->dns_user_hosts_file;
 }
@@ -293,6 +399,14 @@ void belle_sip_stack_set_inactive_transport_timeout(belle_sip_stack_t *stack, in
 	stack->inactive_transport_timeout=seconds;
 }
 
+void belle_sip_stack_set_unreliable_connection_timeout(belle_sip_stack_t *stack, int seconds){
+	stack->unreliable_transport_timeout = seconds;
+}
+
+int belle_sip_stack_get_unreliable_connection_timeout(const belle_sip_stack_t *stack){
+	return stack->unreliable_transport_timeout;
+}
+
 void belle_sip_stack_set_default_dscp(belle_sip_stack_t *stack, int dscp){
 	stack->dscp=dscp;
 }
@@ -315,3 +429,45 @@ int belle_sip_stack_content_encoding_available(belle_sip_stack_t *stack, const c
 GET_SET_STRING(belle_sip_stack,http_proxy_host)
 GET_SET_INT(belle_sip_stack,http_proxy_port, int)
 
+void  belle_sip_set_log_handler(belle_sip_log_function_t func) {
+	bctbx_set_log_handler_for_domain(func, BCTBX_LOG_DOMAIN);
+}
+
+void belle_sip_stack_enable_reconnect_to_primary_asap(belle_sip_stack_t *stack, int enabled) {
+	stack->reconnect_to_primary_asap=enabled;
+}
+
+int belle_sip_stack_reconnect_to_primary_asap_enabled(const belle_sip_stack_t *stack) {
+	return stack->reconnect_to_primary_asap;
+}
+
+void belle_sip_stack_set_digest_authentication_policy(belle_sip_stack_t *stack, belle_sip_digest_authentication_policy_t *policy){
+	SET_OBJECT_PROPERTY(stack, digest_auth_policy, policy);
+}
+
+const belle_sip_digest_authentication_policy_t *belle_sip_stack_get_digest_authentication_policy(const belle_sip_stack_t *stack){
+	return stack->digest_auth_policy;
+}
+
+int belle_sip_stack_check_digest_compatibility(const belle_sip_stack_t *stack, const belle_sip_header_www_authenticate_t *authenticate){
+	const char *algo = belle_sip_header_www_authenticate_get_algorithm(authenticate);
+
+	if (!stack->digest_auth_policy->allow_md5){
+		if (!algo || strcasecmp(algo, "MD5") == 0){
+			belle_sip_warning("Rejecting MD5 digest authentication, not allowed per configuration.");
+			return -1;
+		}
+	}
+	if (!stack->digest_auth_policy->allow_no_qop){
+		const char *qop_value = belle_sip_header_www_authenticate_get_qop_first(authenticate);
+		if (!qop_value || strcasecmp(qop_value, "auth") != 0){
+			belle_sip_warning("Rejecting digest authentication with qop != auth, not allowed per configuration.");
+			return -1;
+		}
+	}
+	return 0;
+}
+
+void belle_sip_stack_set_client_bind_port(belle_sip_stack_t *stack, int port){
+	stack->test_bind_port = port;
+}
