@@ -1,5 +1,25 @@
+/*
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "wakelock_internal.h"
 #include "belle-sip/utils.h"
+#include "bctoolbox/port.h"
 #include <pthread.h>
 
 struct _WakeLock {
@@ -10,27 +30,49 @@ struct _WakeLock {
 	jmethodID newWakeLockID;
 	jmethodID acquireID;
 	jmethodID releaseID;
+	int numberOfWakelocks;
+	int numberOfWakelocksAcquired;
 };
 
 typedef struct _WakeLock WakeLock;
 
 static WakeLock ctx = {
 	.jvm = NULL,
-	.powerManager = NULL
+	.powerManager = NULL,
+	.numberOfWakelocks = 0,
+	.numberOfWakelocksAcquired = 0
 };
 
+bctbx_mutex_t wakeLockInitMutex = PTHREAD_MUTEX_INITIALIZER;
+bctbx_mutex_t wakeLockMutex = PTHREAD_MUTEX_INITIALIZER;
+
 static JNIEnv *get_jni_env(void);
 static void jni_key_cleanup(void *data);
 
-void belle_sip_wake_lock_init(JNIEnv *env, jobject pm) {
+static void belle_sip_set_jvm(JNIEnv *env) {
 	if (ctx.jvm == NULL) {
+		(*env)->GetJavaVM(env, &ctx.jvm);
+		pthread_key_create(&ctx.jniEnvKey, jni_key_cleanup);
+		belle_sip_message("belle_sip_set_jvm(): initialization succeed");
+	} else {
+		belle_sip_warning("belle_sip_set_jvm(): the JNIEnv has already been initialized");
+	}
+}
+
+void belle_sip_wake_lock_init(JNIEnv *env, jobject pm) {
+	bctbx_mutex_lock(&wakeLockInitMutex);
+	if (ctx.jvm == NULL) {
+		belle_sip_set_jvm(env);
+	}
+
+	ctx.numberOfWakelocks++;
+	belle_sip_debug("bellesip_wake_lock : Number of wake locks = %d", ctx.numberOfWakelocks);
+
+	if (ctx.powerManager == NULL) {
 		jclass powerManagerClass;
 		jclass wakeLockClass;
 		jfieldID fieldID;
 
-		(*env)->GetJavaVM(env, &ctx.jvm);
-		pthread_key_create(&ctx.jniEnvKey, jni_key_cleanup);
-
 		powerManagerClass = (*env)->FindClass(env, "android/os/PowerManager");
 		wakeLockClass = (*env)->FindClass(env, "android/os/PowerManager$WakeLock");
 		fieldID = (*env)->GetStaticFieldID(env, powerManagerClass, "PARTIAL_WAKE_LOCK", "I");
@@ -40,21 +82,30 @@ void belle_sip_wake_lock_init(JNIEnv *env, jobject pm) {
 		ctx.acquireID = (*env)->GetMethodID(env, wakeLockClass, "acquire", "()V");
 		ctx.releaseID = (*env)->GetMethodID(env, wakeLockClass, "release", "()V");
 
+		ctx.powerManager = (*env)->NewGlobalRef(env, pm);
 		belle_sip_message("bellesip_wake_lock_init(): initialization succeed");
 	} else {
 		belle_sip_warning("bellesip_wake_lock_init(): the wakelock system has already been initialized");
 	}
-	if (ctx.powerManager == NULL) {
-		ctx.powerManager = (*env)->NewGlobalRef(env, pm);
-	}
+	bctbx_mutex_unlock(&wakeLockInitMutex);
 }
 
 void belle_sip_wake_lock_uninit(JNIEnv *env) {
-	ctx.jvm = NULL;
-	if(ctx.powerManager != NULL) {
-		(*env)->DeleteGlobalRef(env, ctx.powerManager);
-		ctx.powerManager = NULL;
+	bctbx_mutex_lock(&wakeLockInitMutex);
+	if (ctx.powerManager != NULL) {
+		ctx.numberOfWakelocks--;
+		belle_sip_debug("bellesip_wake_lock : Number of wake locks = %d", ctx.numberOfWakelocks);
+		if(ctx.numberOfWakelocks == 0){
+			(*env)->DeleteGlobalRef(env, ctx.powerManager);
+			ctx.powerManager = NULL;
+			belle_sip_message("bellesip_wake_lock_uninit(): uninitialization succeed");
+		} else if(ctx.numberOfWakelocks < 0){
+			belle_sip_warning("bellesip_wake_lock_uninit(): There is atleast one extra uninit()");
+		}
+	} else {
+		belle_sip_warning("bellesip_wake_lock_uninit(): the wakelock system has already been uninitialized");
 	}
+	bctbx_mutex_unlock(&wakeLockInitMutex);
 }
 
 /**
@@ -102,6 +153,7 @@ static JNIEnv *get_jni_env(void) {
 }
 
 unsigned long wake_lock_acquire(const char *tag) {
+	bctbx_mutex_lock(&wakeLockMutex);
 	if(ctx.jvm != NULL && ctx.powerManager != NULL) {
 		JNIEnv *env;
 		if((env = get_jni_env())) {
@@ -110,9 +162,15 @@ unsigned long wake_lock_acquire(const char *tag) {
 			(*env)->DeleteLocalRef(env, tagString);
 			if(lock != NULL) {
 				(*env)->CallVoidMethod(env, lock, ctx.acquireID);
-				lock = (*env)->NewGlobalRef(env, lock);
-				belle_sip_message("bellesip_wake_lock_acquire(): Android wake lock acquired [ref=%p]", (void *)lock);
-				return (unsigned long)lock;
+				jobject lock2 = (*env)->NewGlobalRef(env, lock);
+				(*env)->DeleteLocalRef(env, lock);
+				ctx.numberOfWakelocksAcquired++;
+				belle_sip_debug("bellesip_wake_lock : Number of wake locks ACQUIRED = %d", ctx.numberOfWakelocksAcquired);
+				belle_sip_message("bellesip_wake_lock_acquire(): Android wake lock [%s] acquired [ref=%p]", tag, (void *)lock2);
+				unsigned long lock2value = (unsigned long)lock2;
+				belle_sip_message("bellesip_wake_lock_acquire(): cast long of wakelock %lu", lock2value);
+				bctbx_mutex_unlock(&wakeLockMutex);
+				return (unsigned long)lock2;
 			} else {
 				belle_sip_message("bellesip_wake_lock_acquire(): wake lock creation failed");
 			}
@@ -130,6 +188,7 @@ unsigned long wake_lock_acquire(const char *tag) {
 
 
 void wake_lock_release(unsigned long id) {
+	bctbx_mutex_lock(&wakeLockMutex);
 	if(ctx.jvm != NULL && ctx.powerManager != NULL) {
 		if(id != 0) {
 			jobject lock = (jobject)id;
@@ -137,6 +196,8 @@ void wake_lock_release(unsigned long id) {
 			if((env = get_jni_env())) {
 				(*env)->CallVoidMethod(env, lock, ctx.releaseID);
 				belle_sip_message("bellesip_wake_lock_release(): Android wake lock released [ref=%p]", (void *)lock);
+				ctx.numberOfWakelocksAcquired--;
+				belle_sip_debug("bellesip_wake_lock : Number of wake locks ACQUIRED = %d", ctx.numberOfWakelocksAcquired);
 				(*env)->DeleteGlobalRef(env, lock);
 			} else {
 				belle_sip_error("bellesip_wake_lock_release(): cannot attach current thread to the JVM");
@@ -148,4 +209,5 @@ void wake_lock_release(unsigned long id) {
 		else
 			belle_sip_error("bellesip_wake_lock_release(): cannot release wake lock. No PowerManager found");
 	}
+	bctbx_mutex_unlock(&wakeLockMutex);
 }
