@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +41,75 @@ static void test_authentication(void) {
 	belle_sip_object_unref(authorization);
 }
 
+static void test_authentication_sha256(void) {
+	const char *l_raw_header = "WWW-Authenticate: Digest "
+	                           "algorithm=SHA-256, realm=\"http-auth@example.org\", opaque=\"FQhe/qaU925kfnzjCev0ciny7QMkPqMAFRtzCUYo5tdS\","
+	                           " qop=\"auth,auth-int\", nonce=\"7ypf/xlj9XXwfDPEoM4URrv/xwf94BcCAzFZH4GiTo0v\"";
+	char ha1[65];
+	size_t size = sizeof(ha1) / sizeof(char);
+	belle_sip_header_www_authenticate_t *www_authenticate = belle_sip_header_www_authenticate_parse(l_raw_header);
+	belle_sip_header_authorization_t *authorization = belle_sip_auth_helper_create_authorization(www_authenticate);
+	belle_sip_header_authorization_set_uri(authorization, belle_sip_uri_parse("sip:sip.linphone.org"));
+	belle_sip_header_authorization_set_nonce_count(authorization, 1);
+	belle_sip_header_authorization_set_qop(authorization, "auth");
+	belle_sip_header_authorization_set_cnonce(authorization, "8302210f"); /*for testing purpose*/
+	const char *algo = belle_sip_header_authorization_get_algorithm(authorization);
+	BC_ASSERT_EQUAL(0, belle_sip_auth_helper_compute_ha1_for_algorithm("Mufasa", "http-auth@example.org", "Circle of Life", ha1, size, algo), int, "%d");
+	BC_ASSERT_EQUAL(0, belle_sip_auth_helper_fill_authorization(authorization, "REGISTER", ha1), int, "%d");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_qop(authorization), "auth");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_response(authorization), "91253bea2bbc3b34a5dac192c672fb7fff1ae6d657207b6719842827a509a501");
+	BC_ASSERT_EQUAL(belle_sip_header_authorization_get_nonce_count(authorization), 1, int, "%d");
+	belle_sip_object_unref(www_authenticate);
+	belle_sip_object_unref(authorization);
+}
+
+/* test patterns from RFC7616 section 3.9.1 */
+static void test_authentication_sha256_rfc7616(void) {
+	const char *l_raw_header = "WWW-Authenticate: Digest "
+	                           "algorithm=SHA-256, realm=\"http-auth@example.org\", opaque=\"FQhe/qaU925kfnzjCev0ciny7QMkPqMAFRtzCUYo5tdS\","
+	                           " qop=\"auth,auth-int\", nonce=\"7ypf/xlj9XXwfDPEoM4URrv/xwf94BcCAzFZH4GiTo0v\"";
+	char ha1[65];
+	size_t size = sizeof(ha1) / sizeof(char);
+	belle_sip_header_www_authenticate_t *www_authenticate = belle_sip_header_www_authenticate_parse(l_raw_header);
+	belle_http_header_authorization_t *authorization = belle_http_auth_helper_create_authorization(www_authenticate);
+	belle_http_header_authorization_set_uri(authorization, belle_generic_uri_parse("/dir/index.html"));
+	belle_sip_header_authorization_set_nonce_count(BELLE_SIP_HEADER_AUTHORIZATION(authorization), 1);
+	belle_sip_header_authorization_set_qop(BELLE_SIP_HEADER_AUTHORIZATION(authorization), "auth");
+	belle_sip_header_authorization_set_cnonce(BELLE_SIP_HEADER_AUTHORIZATION(authorization), "f2/wE4q74E6zIJEtWaHKaf5wv/H5QzzpXusqGemxURZJ");
+	const char *algo = belle_sip_header_authorization_get_algorithm(BELLE_SIP_HEADER_AUTHORIZATION(authorization));
+	BC_ASSERT_EQUAL(0, belle_sip_auth_helper_compute_ha1_for_algorithm("Mufasa", "http-auth@example.org", "Circle of Life", ha1, size, algo), int, "%d");
+	BC_ASSERT_EQUAL(0, belle_sip_auth_helper_fill_authorization(BELLE_SIP_HEADER_AUTHORIZATION(authorization), "GET", ha1), int, "%d");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_qop(BELLE_SIP_HEADER_AUTHORIZATION(authorization)), "auth");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_response(BELLE_SIP_HEADER_AUTHORIZATION(authorization)), "753927fa0e85d155564e2e272a28d1802ca10daf4496794697cf8db5856cb6c1");
+	BC_ASSERT_EQUAL(belle_sip_header_authorization_get_nonce_count(BELLE_SIP_HEADER_AUTHORIZATION(authorization)), 1, int, "%d");
+	belle_sip_object_unref(www_authenticate);
+	belle_sip_object_unref(authorization);
+}
+
+static void test_authentication_md5_rfc7616(void) {
+	const char *l_raw_header = "WWW-Authenticate: Digest "
+	                           "algorithm=MD5, realm=\"http-auth@example.org\", opaque=\"FQhe/qaU925kfnzjCev0ciny7QMkPqMAFRtzCUYo5tdS\","
+	                           " qop=\"auth,auth-int\", nonce=\"7ypf/xlj9XXwfDPEoM4URrv/xwf94BcCAzFZH4GiTo0v\"";
+	char ha1[33];
+	size_t size = sizeof(ha1) / sizeof(char);
+	belle_sip_header_www_authenticate_t *www_authenticate = belle_sip_header_www_authenticate_parse(l_raw_header);
+	belle_http_header_authorization_t *authorization = belle_http_auth_helper_create_authorization(www_authenticate);
+	belle_http_header_authorization_set_uri(authorization, belle_generic_uri_parse("/dir/index.html"));
+	belle_sip_header_authorization_set_nonce_count(BELLE_SIP_HEADER_AUTHORIZATION(authorization), 1);
+	belle_sip_header_authorization_set_qop(BELLE_SIP_HEADER_AUTHORIZATION(authorization), "auth");
+	belle_sip_header_authorization_set_cnonce(BELLE_SIP_HEADER_AUTHORIZATION(authorization), "f2/wE4q74E6zIJEtWaHKaf5wv/H5QzzpXusqGemxURZJ");
+	const char *algo = belle_sip_header_authorization_get_algorithm(BELLE_SIP_HEADER_AUTHORIZATION(authorization));
+	BC_ASSERT_EQUAL(0, belle_sip_auth_helper_compute_ha1_for_algorithm("Mufasa", "http-auth@example.org", "Circle of Life", ha1, size, algo), int, "%d");
+	BC_ASSERT_EQUAL(0, belle_sip_auth_helper_fill_authorization(BELLE_SIP_HEADER_AUTHORIZATION(authorization), "GET", ha1), int, "%d");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_qop(BELLE_SIP_HEADER_AUTHORIZATION(authorization)), "auth");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_response(BELLE_SIP_HEADER_AUTHORIZATION(authorization)), "8ca523f5e9506fed4657c9700eebdbec");
+	BC_ASSERT_EQUAL(belle_sip_header_authorization_get_nonce_count(BELLE_SIP_HEADER_AUTHORIZATION(authorization)), 1, int, "%d");
+	belle_sip_object_unref(www_authenticate);
+	belle_sip_object_unref(authorization);
+}
+
+
+
 static void test_authentication_qop_auth(void) {
 	const char* l_raw_header = "WWW-Authenticate: Digest "
 				"algorithm=MD5, realm=\"sip.linphone.org\", opaque=\"1bc7f9097684320\","
@@ -196,6 +266,9 @@ static void test_certificate_fingerprint(void) {
 test_t authentication_helper_tests[] = {
 	TEST_NO_TAG("Proxy-Authenticate", test_proxy_authentication),
 	TEST_NO_TAG("WWW-Authenticate", test_authentication),
+	TEST_NO_TAG("WWW-Authenticate-sha", test_authentication_sha256),
+	TEST_NO_TAG("WWW-Authenticate-sha256 RFC7616 patterns", test_authentication_sha256_rfc7616),
+	TEST_NO_TAG("WWW-Authenticate-MD5 RFC7616 patterns", test_authentication_md5_rfc7616),
 	TEST_NO_TAG("WWW-Authenticate (with qop)", test_authentication_qop_auth),
 	TEST_NO_TAG("generate and parse self signed certificates", test_generate_and_parse_certificates),
 	TEST_NO_TAG("generate certificate fingerprint", test_certificate_fingerprint)
