@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-	Copyright (C) 2010  Belledonne Communications SARL
-
-	This program is free software: you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle-sip/belle-sip.h"
 #include "belle-sip/object.h"
@@ -303,10 +304,22 @@ static void test_presence_marshal(void) {
 	}
 	desc = belle_sip_object_to_string(mpbh);
 	BC_ASSERT_PTR_NOT_NULL(desc);
+
 	if (desc != NULL) {
-		BC_ASSERT_EQUAL((unsigned int)strlen(desc), 4676, unsigned int, "%u");
+		BC_ASSERT_EQUAL((unsigned int)strlen(desc), 4688, unsigned int, "%u");
+		belle_sip_message("Body is [%s]",desc);
 	}
+	belle_sip_multipart_body_handler_t *memory_mpbh = belle_sip_multipart_body_handler_new_from_buffer(desc, strlen(desc), belle_sip_multipart_body_handler_get_boundary(mpbh));
+
+	const bctbx_list_t * parts = belle_sip_multipart_body_handler_get_parts(memory_mpbh);
+	for (i = 0; i < 7 ; i++) {
+		belle_sip_body_handler_t *part = (belle_sip_body_handler_t *)parts->data;
+		BC_ASSERT_EQUAL((int)belle_sip_body_handler_get_size(part), (int)strlen(parts_content[i]), int, "%i");
+		parts = parts->next;
+	}
+
 	belle_sip_object_unref(mpbh);
+	belle_sip_object_unref(memory_mpbh);
 }
 
 static void test_compressed_body(void) {
