@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-	Copyright (C) 2010  Belledonne Communications SARL
-
-	This program is free software: you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2021 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle-sip/belle-sip.h"
 #include "belle_sip_internal.h"
@@ -23,16 +24,19 @@
 #include "register_tester.h"
 
 
-const char *test_domain="sip2.linphone.org";
-const char *auth_domain="sip.linphone.org";
+const int RANDOM_PORT = -1;
+#define AUTH_DOMAIN "sip.example.org"
+const char *test_domain="sipopen.example.org";
+const char *auth_domain=AUTH_DOMAIN;
 const char *client_auth_domain="client.example.org";
-const char *client_auth_outbound_proxy="sips:sip2.linphone.org:5063";
+const char *client_auth_outbound_proxy="sips:sip.example.org:5063";
 const char *no_server_running_here="sip:test.linphone.org:3;transport=tcp";
 const char *no_response_here="sip:78.220.48.77:3;transport=%s";
 const char *test_domain_tls_to_tcp="sip:sip2.linphone.org:5060;transport=tls";
 const char *test_http_proxy_addr="sip.linphone.org";
 const char *test_with_wrong_cname="sips:rototo.com;maddr=91.121.209.194";
-int test_http_proxy_port = 3128 ;
+int test_http_proxy_port = 3128;
+
 
 static int is_register_ok;
 static int number_of_challenge;
@@ -46,7 +50,6 @@ belle_sip_request_t* authorized_request;
 belle_sip_listener_callbacks_t listener_callbacks;
 belle_sip_listener_t *listener;
 
-
 static void process_dialog_terminated(void *user_ctx, const belle_sip_dialog_terminated_event_t *event){
 	BELLESIP_UNUSED(user_ctx);
 	BELLESIP_UNUSED(event);
@@ -168,7 +171,7 @@ const char* belle_sip_tester_private_key =
 const char* belle_sip_tester_private_key_passwd="secret";
 
 
-const char *belle_sip_tester_root_ca = 
+const char *belle_sip_tester_root_ca =
 "-----BEGIN CERTIFICATE-----\n"
 "MIIDRjCCAq+gAwIBAgIJAJ3nFcA7qFrOMA0GCSqGSIb3DQEBBQUAMIG7MQswCQYD\n"
 "VQQGEwJGUjETMBEGA1UECAwKU29tZS1TdGF0ZTERMA8GA1UEBwwIR3Jlbm9ibGUx\n"
@@ -212,58 +215,96 @@ const char *belle_sip_tester_root_ca =
 "6wwCURQtjr0W4MHfRnXnJK3s9EK0hZNwEGe6nQY1ShjTK3rMUUKhemPR5ruhxSvCNr4TDea9Y355\n"
 "e6cJDUCrat2PisP29owaQgVR1EX1n6diIWgVIEM8med8vSTYqZEXc4g/VhsxOBi0cQ+azcgOno4u\n"
 "G+GMmIPLHzHxREzGBHNJdmAPx/i9F4BrLunMTA5amnkPIAou1Z5jJh5VkpTYghdae9C8x49OhgQ=\n"
-"-----END CERTIFICATE-----\n";
+"-----END CERTIFICprocess_auth_requestedATE-----\n";
 
-static void process_auth_requested(void *user_ctx, belle_sip_auth_event_t *event){
+static void process_auth_requested(void *user_ctx, belle_sip_auth_event_t *event) {
 	BELLESIP_UNUSED(user_ctx);
 	if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_HTTP_DIGEST) {
 		const char *username = belle_sip_auth_event_get_username(event);
 		const char *realm = belle_sip_auth_event_get_realm(event);
 		belle_sip_message("process_auth_requested requested for [%s@%s]"
-				,username?username:""
-				,realm?realm:"");
-		belle_sip_auth_event_set_passwd(event,"secret");
+		                  , username ? username : ""
+		                  , realm ? realm : "");
+		belle_sip_auth_event_set_passwd(event, "secret");
 	} else if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_TLS) {
 		const char *distinguished_name = NULL;
-		belle_sip_certificates_chain_t* cert = belle_sip_certificates_chain_parse(belle_sip_tester_client_cert,strlen(belle_sip_tester_client_cert),BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
-		belle_sip_signing_key_t* key = belle_sip_signing_key_parse(belle_sip_tester_private_key,strlen(belle_sip_tester_private_key),belle_sip_tester_private_key_passwd);
-		belle_sip_auth_event_set_client_certificates_chain(event,cert);
-		belle_sip_auth_event_set_signing_key(event,key);
+		belle_sip_certificates_chain_t *cert = belle_sip_certificates_chain_parse(belle_sip_tester_client_cert, strlen(belle_sip_tester_client_cert), BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
+		belle_sip_signing_key_t *key = belle_sip_signing_key_parse(belle_sip_tester_private_key, strlen(belle_sip_tester_private_key), belle_sip_tester_private_key_passwd);
+		belle_sip_auth_event_set_client_certificates_chain(event, cert);
+		belle_sip_auth_event_set_signing_key(event, key);
 		distinguished_name = belle_sip_auth_event_get_distinguished_name(event);
-		belle_sip_message("process_auth_requested requested for  DN[%s]",distinguished_name?distinguished_name:"");
+		belle_sip_message("process_auth_requested requested for  DN[%s]", distinguished_name ? distinguished_name : "");
 
 	} else {
 		belle_sip_error("Unexpected auth mode");
 	}
 }
 
+static void process_auth_requested_for_algorithm(void *user_ctx, belle_sip_auth_event_t *event) {
+	const char **client;
+	client = (const char **) user_ctx; //*client is algorithm of client, *(client+1) is password haché
+	if (*client == NULL)
+		*client = "MD5";
+	if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_HTTP_DIGEST) {
+		const char *username = belle_sip_auth_event_get_username(event);
+		const char *realm = belle_sip_auth_event_get_realm(event);
+		belle_sip_message("process_auth_requested requested for [%s@%s]"
+		                  , username ? username : ""
+		                  , realm ? realm : "");
+		/* Default algorithm is MD5 if it's NULL. If algorithm of client = algorithm of server (event->algorithm), set ha1 or passwd. */
+		if (((event->algorithm) && (!strcmp(*client, event->algorithm))) || ((event->algorithm == NULL) && (!strcmp(*client, "MD5")))) {
+			if (*(client + 1))
+				belle_sip_auth_event_set_ha1(event, *(client + 1));
+			else
+				belle_sip_auth_event_set_passwd(event, "secret");
+		}
+	} else if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_TLS) {
+		const char *distinguished_name = NULL;
+		belle_sip_certificates_chain_t *cert = belle_sip_certificates_chain_parse(belle_sip_tester_client_cert, strlen(belle_sip_tester_client_cert), BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
+		belle_sip_signing_key_t *key = belle_sip_signing_key_parse(belle_sip_tester_private_key, strlen(belle_sip_tester_private_key), belle_sip_tester_private_key_passwd);
+		belle_sip_auth_event_set_client_certificates_chain(event, cert);
+		belle_sip_auth_event_set_signing_key(event, key);
+		distinguished_name = belle_sip_auth_event_get_distinguished_name(event);
+		belle_sip_message("process_auth_requested requested for  DN[%s]", distinguished_name ? distinguished_name : "");
+
+	} else {
+		belle_sip_error("Unexpected auth mode");
+	}
+}
+
+static const char *listener_user_data[2] = {NULL, NULL};
+
 int register_before_all(void) {
 	belle_sip_listening_point_t *lp;
-	stack=belle_sip_stack_new(NULL);
-	lp=belle_sip_stack_create_listening_point(stack,"0.0.0.0",7060,"UDP");
-	prov=belle_sip_stack_create_provider(stack,lp);
+	stack = belle_sip_stack_new(NULL);
 
-	lp=belle_sip_stack_create_listening_point(stack,"0.0.0.0",7060,"TCP");
-	belle_sip_provider_add_listening_point(prov,lp);
-	lp=belle_sip_stack_create_listening_point(stack,"0.0.0.0",7061,"TLS");
+	belle_sip_tester_set_dns_host_file(stack);
+
+	lp = belle_sip_stack_create_listening_point(stack, "0.0.0.0", RANDOM_PORT, "UDP");
+	prov = belle_sip_stack_create_provider(stack, lp);
+
+	lp = belle_sip_stack_create_listening_point(stack, "0.0.0.0", RANDOM_PORT, "TCP");
+	belle_sip_provider_add_listening_point(prov, lp);
+	lp = belle_sip_stack_create_listening_point(stack, "0.0.0.0", RANDOM_PORT, "TLS");
 	if (lp) {
-		belle_tls_crypto_config_t *crypto_config=belle_tls_crypto_config_new();
-	
+		belle_tls_crypto_config_t *crypto_config = belle_tls_crypto_config_new();
+
 		belle_tls_crypto_config_set_root_ca_data(crypto_config, belle_sip_tester_root_ca);
 		belle_sip_tls_listening_point_set_crypto_config(BELLE_SIP_TLS_LISTENING_POINT(lp), crypto_config);
-		belle_sip_provider_add_listening_point(prov,lp);
+		belle_sip_provider_add_listening_point(prov, lp);
 		belle_sip_object_unref(crypto_config);
 	}
 
-	listener_callbacks.process_dialog_terminated=process_dialog_terminated;
-	listener_callbacks.process_io_error=process_io_error;
-	listener_callbacks.process_request_event=process_request_event;
-	listener_callbacks.process_response_event=process_response_event;
-	listener_callbacks.process_timeout=process_timeout;
-	listener_callbacks.process_transaction_terminated=process_transaction_terminated;
-	listener_callbacks.process_auth_requested=process_auth_requested;
-	listener_callbacks.listener_destroyed=NULL;
-	listener=belle_sip_listener_create_from_callbacks(&listener_callbacks,NULL);
+	listener_callbacks.process_dialog_terminated = process_dialog_terminated;
+	listener_callbacks.process_io_error = process_io_error;
+	listener_callbacks.process_request_event = process_request_event;
+	listener_callbacks.process_response_event = process_response_event;
+	listener_callbacks.process_timeout = process_timeout;
+	listener_callbacks.process_transaction_terminated = process_transaction_terminated;
+	listener_callbacks.process_auth_requested = process_auth_requested_for_algorithm;
+	listener_callbacks.listener_destroyed = NULL;
+
+	listener = belle_sip_listener_create_from_callbacks(&listener_callbacks, (void *)listener_user_data);
 	return 0;
 }
 
@@ -311,30 +352,56 @@ void unregister_user(belle_sip_stack_t * stack
 	belle_sip_provider_remove_sip_listener(prov,l);
 }
 
-belle_sip_request_t* try_register_user_at_domain(belle_sip_stack_t * stack
+static belle_sip_request_t* create_registration_request(belle_sip_stack_t * stack
 	,belle_sip_provider_t *prov
 	,const char *transport
-	,int use_transaction
 	,const char* username
-	,const char* domain
-	,const char* outbound_proxy
-	,int success_expected) {
-	belle_sip_request_t *req,*copy;
+	,const char* domain) {
+	belle_sip_request_t *req;
 	char identity[256];
 	char uri[256];
-	int i;
-	char *outbound=NULL;
-	int do_manual_retransmissions = FALSE;
 
 	number_of_challenge=0;
 	if (transport)
 		snprintf(uri,sizeof(uri),"sip:%s;transport=%s",domain,transport);
-	else snprintf(uri,sizeof(uri),"sip:%s",domain);
+		else snprintf(uri,sizeof(uri),"sip:%s",domain);
 
-	if (transport && strcasecmp("tls",transport)==0 && belle_sip_provider_get_listening_point(prov,"tls")==NULL){
-		belle_sip_error("No TLS support, test skipped.");
-		return NULL;
-	}
+			if (transport && strcasecmp("tls",transport)==0 && belle_sip_provider_get_listening_point(prov,"tls")==NULL){
+				belle_sip_error("No TLS support, test skipped.");
+				return NULL;
+			}
+
+	snprintf(identity,sizeof(identity),"Tester <sip:%s@%s>",username,domain);
+	req=belle_sip_request_create(
+								 belle_sip_uri_parse(uri),
+								 "REGISTER",
+								 belle_sip_provider_create_call_id(prov),
+								 belle_sip_header_cseq_create(20,"REGISTER"),
+								 belle_sip_header_from_create2(identity,BELLE_SIP_RANDOM_TAG),
+								 belle_sip_header_to_create2(identity,NULL),
+								 belle_sip_header_via_new(),
+								 70);
+	belle_sip_object_ref(req);
+	is_register_ok=0;
+	io_error_count=0;
+	using_transaction=0;
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_expires_create(600)));
+	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_contact_new()));
+
+	return req;
+}
+
+static void execute_registration(belle_sip_stack_t * stack,
+								belle_sip_provider_t *prov,
+								belle_sip_client_transaction_t *trans,
+								belle_sip_request_t *req,
+								const char *transport,
+								const char* outbound_proxy,
+								int success_expected){
+	int do_manual_retransmissions = FALSE;
+	int use_transaction = trans ? 1 : 0;
+	int i;
+	char *outbound=NULL;
 
 	if (outbound_proxy){
 		if (strstr(outbound_proxy,"sip:")==NULL && strstr(outbound_proxy,"sips:")==NULL){
@@ -342,27 +409,9 @@ belle_sip_request_t* try_register_user_at_domain(belle_sip_stack_t * stack
 		}else outbound=belle_sip_strdup(outbound_proxy);
 	}
 
-	snprintf(identity,sizeof(identity),"Tester <sip:%s@%s>",username,domain);
-	req=belle_sip_request_create(
-						belle_sip_uri_parse(uri),
-						"REGISTER",
-						belle_sip_provider_create_call_id(prov),
-						belle_sip_header_cseq_create(20,"REGISTER"),
-						belle_sip_header_from_create2(identity,BELLE_SIP_RANDOM_TAG),
-						belle_sip_header_to_create2(identity,NULL),
-						belle_sip_header_via_new(),
-						70);
-	belle_sip_object_ref(req);
-	is_register_ok=0;
-	io_error_count=0;
-	using_transaction=0;
-	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_expires_create(600)));
-	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_contact_new()));
-	copy=(belle_sip_request_t*)belle_sip_object_ref(belle_sip_object_clone((belle_sip_object_t*)req));
 	belle_sip_provider_add_sip_listener(prov,l=BELLE_SIP_LISTENER(listener));
-	if (use_transaction){
-		belle_sip_client_transaction_t *t=belle_sip_provider_create_client_transaction(prov,req);
-		belle_sip_client_transaction_send_request_to(t,outbound?belle_sip_uri_parse(outbound):NULL);
+	if (trans){
+		belle_sip_client_transaction_send_request_to(trans,outbound?belle_sip_uri_parse(outbound):NULL);
 	}else{
 		belle_sip_provider_send_request(prov,req);
 		do_manual_retransmissions = (transport == NULL) || (strcasecmp(transport,"udp") == 0);
@@ -377,9 +426,30 @@ belle_sip_request_t* try_register_user_at_domain(belle_sip_stack_t * stack
 	BC_ASSERT_EQUAL(is_register_ok,success_expected,int,"%d");
 	if (success_expected) BC_ASSERT_EQUAL(using_transaction,use_transaction,int,"%d");
 
-	belle_sip_object_unref(req);
 	belle_sip_provider_remove_sip_listener(prov,l);
 	if (outbound) belle_sip_free(outbound);
+}
+
+
+belle_sip_request_t* try_register_user_at_domain(belle_sip_stack_t * stack
+	,belle_sip_provider_t *prov
+	,const char *transport
+	,int use_transaction
+	,const char* username
+	,const char* domain
+	,const char* outbound_proxy
+	,int success_expected) {
+	belle_sip_request_t *req,*copy = NULL;
+
+	req = create_registration_request(stack, prov, transport, username, domain);
+	if (req)
+	{
+		copy = (belle_sip_request_t*)belle_sip_object_ref(belle_sip_object_clone((belle_sip_object_t*)req));
+		belle_sip_client_transaction_t *t = use_transaction ? belle_sip_provider_create_client_transaction(prov,req) : NULL;
+		execute_registration(stack, prov, t, req, transport, outbound_proxy, success_expected);
+		belle_sip_object_unref(req);
+	}
+
 	return copy;
 }
 
@@ -403,6 +473,25 @@ belle_sip_request_t* register_user(belle_sip_stack_t * stack
 	return register_user_at_domain(stack,prov,transport,use_transaction,username,test_domain,outbound);
 }
 
+belle_sip_client_transaction_t* register_user_with_transaction(belle_sip_stack_t * stack
+					,belle_sip_provider_t *prov
+					,const char *transport
+					,const char* username
+					,const char* outbound_proxy) {
+	belle_sip_request_t *req;
+	belle_sip_client_transaction_t *t = NULL;
+
+	req = create_registration_request(stack, prov, transport, username, test_domain);
+	if (req)
+	{
+		t = belle_sip_provider_create_client_transaction(prov, req);
+		belle_sip_object_ref(t);
+		execute_registration(stack, prov, t, req, transport, outbound_proxy, 1);
+	}
+
+	return t;
+}
+
 static void register_with_outbound(const char *transport, int use_transaction,const char* outbound ) {
 	belle_sip_request_t *req;
 
@@ -468,7 +557,7 @@ static void stateful_register_tls(void){
 
 static void stateful_register_tls_with_wrong_cname(void){
 	belle_sip_request_t *req;
-	
+
 	req = try_register_user_at_domain(stack, prov, "tls", 1, "tester",test_domain, test_with_wrong_cname, 0);
 	if (req) belle_sip_object_unref(req);
 }
@@ -608,10 +697,10 @@ static void test_channel_moving_to_error_and_cleaned(void){
 		belle_sip_client_transaction_t *tr;
 		char identity[128];
 		char uri[128];
-		
+
 		belle_sip_listening_point_clean_channels(lp);
 		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
-		
+
 		snprintf(identity,sizeof(identity),"Tester <sip:%s@%s>","bellesip",test_domain);
 		snprintf(uri,sizeof(uri),"sip:%s",test_domain);
 		req = belle_sip_request_create(
@@ -634,7 +723,7 @@ static void test_channel_moving_to_error_and_cleaned(void){
 		/*we just want to verify that it doesn't crash*/
 		belle_sip_stack_sleep(stack, 1000);
 		belle_sip_object_unref(tr);
-		
+
 	}
 }
 
@@ -652,6 +741,37 @@ static void test_register_client_authenticated(void) {
 	if (reg) belle_sip_object_unref(reg);
 }
 
+static void test_register_client_bad_ciphersuites(void) {
+	/* If there is no mbedtls, this test will do nothing. */
+	if(bctbx_ssl_get_implementation_type()==BCTBX_MBEDTLS) {
+		belle_sip_request_t *reg;
+		authorized_request=NULL;
+		belle_sip_tls_listening_point_t *s = BELLE_SIP_TLS_LISTENING_POINT(belle_sip_provider_get_listening_point(prov, "tls"));
+		belle_tls_crypto_config_t *crypto_config = belle_sip_tls_listening_point_get_crypto_config(s);
+
+		belle_sip_listening_point_clean_channels((belle_sip_listening_point_t*)s);
+
+		void *config_ref = crypto_config->ssl_config;
+		int ciphersuites[2] = {bctbx_ssl_get_ciphersuite_id("TLS-RSA-WITH-AES-128-GCM-SHA256"),0};
+
+		bctbx_ssl_config_t *sslcfg = bctbx_ssl_config_new();
+		bctbx_ssl_config_defaults( sslcfg, BCTBX_SSL_IS_CLIENT, BCTBX_SSL_TRANSPORT_STREAM);
+		bctbx_ssl_config_set_authmode(sslcfg, BCTBX_SSL_VERIFY_REQUIRED);
+		bctbx_ssl_config_set_ciphersuites(sslcfg,ciphersuites);
+		crypto_config->ssl_config = bctbx_ssl_config_get_private_config(sslcfg);
+
+		/* This ciphersuite will be rejected by flexisip, so success_expected=0. See tls-ciphers in flexisip. */
+		reg=try_register_user_at_domain(stack, prov, "tls",1,"tester",client_auth_domain,client_auth_outbound_proxy,0);
+		if (authorized_request) {
+			unregister_user(stack,prov,authorized_request,1);
+			belle_sip_object_unref(authorized_request);
+		}
+		if (reg) belle_sip_object_unref(reg);
+		bctbx_ssl_config_free(sslcfg);
+		crypto_config->ssl_config = config_ref;
+	}
+}
+
 static void test_connection_failure(void){
 	belle_sip_request_t *req;
 	io_error_count=0;
@@ -776,12 +896,12 @@ static belle_sip_request_t* send_message_to(belle_sip_request_t *initial_request
 	is_register_ok = 0;
 
 	message_request=belle_sip_request_create(
-								belle_sip_uri_parse("sip:sip.linphone.org;transport=tcp")
+								belle_sip_uri_parse("sip:"AUTH_DOMAIN";transport=tcp")
 								,"MESSAGE"
 								,belle_sip_provider_create_call_id(prov)
 								,belle_sip_header_cseq_create(22,"MESSAGE")
 								,belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(initial_request), belle_sip_header_from_t)
-								,belle_sip_header_to_parse("To: sip:marie@sip.linphone.org")
+								,belle_sip_header_to_parse("To: sip:marie@"AUTH_DOMAIN)
 								,belle_sip_header_via_new()
 								,70);
 	belle_sip_message_add_header(BELLE_SIP_MESSAGE(message_request),BELLE_SIP_HEADER(belle_sip_header_expires_create(600)));
@@ -802,16 +922,15 @@ static belle_sip_request_t* send_message_to(belle_sip_request_t *initial_request
 
 static void reuse_nonce_base(const char* outbound) {
 	belle_sip_request_t *register_request;
-	size_t initial_auth_context_count=0;/*belle_sip_list_size(prov->auth_contexts);*/
 	char outbound_uri[256];
 	/*reset auth context*/
 	prov->auth_contexts = belle_sip_list_free_with_data(prov->auth_contexts,(void(*)(void*))belle_sip_authorization_destroy);
-	
+
 	if (outbound)
 		snprintf(outbound_uri, sizeof(outbound_uri),"sip:%s",outbound);
-	
-	register_request=register_user_at_domain(stack, prov, "tcp",1,"marie",outbound,NULL);
-	
+
+	register_request=register_user_at_domain(stack, prov, "tcp",1,"marie",auth_domain, outbound);
+
 	if (register_request) {
 		belle_sip_header_authorization_t * h = NULL;
 		belle_sip_request_t *message_request;
@@ -829,8 +948,23 @@ static void reuse_nonce_base(const char* outbound) {
 
 		belle_sip_provider_add_sip_listener(prov,BELLE_SIP_LISTENER(reuse_nonce_listener));
 
-		/*currently only one nonce should have been used (the one for the REGISTER)*/
-		BC_ASSERT_EQUAL((unsigned int)belle_sip_list_size(prov->auth_contexts), (unsigned int)initial_auth_context_count+1,unsigned int,"%u");
+		unsigned int number_of_md5_auth_context = 0;
+		unsigned int number_of_sha256_auth_context = 0;
+
+		for (belle_sip_list_t *it = prov->auth_contexts ;it!=NULL ; it=it->next) {
+			belle_sip_authorization_t *auth_context=(authorization_context_t*)it->data;
+			if (!belle_sip_authorization_get_algorithm(auth_context) || strcasecmp(belle_sip_authorization_get_algorithm(auth_context),"MD5"))
+				number_of_md5_auth_context++;
+			else if (strcasecmp(belle_sip_authorization_get_algorithm(auth_context),"SHA256"))
+				number_of_sha256_auth_context++;
+			else
+				belle_sip_error("This test does not support algo [%s]",belle_sip_authorization_get_algorithm(auth_context) );
+		}
+
+		/*currently only one nonce by algo should have been used (the one for the REGISTER)*/
+		BC_ASSERT_LOWER(number_of_md5_auth_context, 1 ,unsigned int,"%u");
+		BC_ASSERT_LOWER(number_of_sha256_auth_context, 1 ,unsigned int,"%u");
+		BC_ASSERT_GREATER((number_of_sha256_auth_context + number_of_md5_auth_context), 1 ,unsigned int,"%u");
 
 		/*this should reuse previous nonce*/
 		message_request=send_message_to(register_request, auth_domain,outbound?belle_sip_uri_parse(outbound_uri):NULL);
@@ -871,7 +1005,7 @@ static void reuse_nonce_base(const char* outbound) {
 
 		/*first nonce created should be reused. this test is only for qop = auth*/
 		message_request=send_message_to(register_request, auth_domain,outbound?belle_sip_uri_parse(outbound_uri):NULL);
-		
+
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 			));
@@ -889,7 +1023,7 @@ static void reuse_nonce_base(const char* outbound) {
 	}
 }
 static void reuse_nonce(void) {
-	reuse_nonce_base("sip.linphone.org");
+	reuse_nonce_base(auth_domain);
 }
 #define NONCE_SIZE  32
 void register_process_request_event(char *nonce, const belle_sip_request_event_t *event) {
@@ -899,11 +1033,11 @@ void register_process_request_event(char *nonce, const belle_sip_request_event_t
 	char *uri_as_string= belle_sip_uri_to_string(belle_sip_request_get_uri(req));
 	belle_sip_response_t * response_msg;
 	belle_sip_server_transaction_t *trans=belle_sip_provider_create_server_transaction(prov, req);
-	
+
 	if (strcasecmp(belle_sip_request_get_method(req), "REGISTER") == 0) {
 		response_code=401;
 	}
-	
+
 	if (	(authorization = belle_sip_message_get_header_by_type(req,belle_sip_header_authorization_t))
 		 || (authorization = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(req,belle_sip_header_proxy_authorization_t)))) {
 		 char ha1[33], ha2[33], response[33];
@@ -922,16 +1056,16 @@ void register_process_request_event(char *nonce, const belle_sip_request_event_t
 			}
 		}
 	}
-	
+
 	belle_sip_random_token((nonce), NONCE_SIZE);
 	response_msg = belle_sip_response_create_from_request(req, response_code);
-	
+
 	if (response_code == 407 || response_code == 401 ) {
 		belle_sip_header_www_authenticate_t *www_authenticate = 401?belle_sip_header_www_authenticate_new():BELLE_SIP_HEADER_WWW_AUTHENTICATE(belle_sip_header_proxy_authenticate_new());
-	
-		
-		belle_sip_header_www_authenticate_set_realm(www_authenticate, "sip.linphone.org");
-		belle_sip_header_www_authenticate_set_domain(www_authenticate, "sip:sip.linphone.org");
+
+
+		belle_sip_header_www_authenticate_set_realm(www_authenticate, AUTH_DOMAIN);
+		belle_sip_header_www_authenticate_set_domain(www_authenticate, "sip:"AUTH_DOMAIN);
 		belle_sip_header_www_authenticate_set_scheme(www_authenticate, "Digest");
 		belle_sip_header_www_authenticate_set_nonce(www_authenticate,nonce);
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response_msg), BELLE_SIP_HEADER(www_authenticate));
@@ -940,7 +1074,7 @@ void register_process_request_event(char *nonce, const belle_sip_request_event_t
 		belle_sip_header_authentication_info_set_next_nonce(authentication_info, nonce);
 		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response_msg), BELLE_SIP_HEADER(authentication_info));
 	}
-	
+
 	belle_sip_server_transaction_send_response(trans,response_msg);
 }
 
@@ -951,7 +1085,7 @@ static void test_register_with_next_nonce(void) {
 	char listening_uri[256];
 	belle_sip_listener_callbacks_t cbs;
 	belle_sip_random_token((nonce), sizeof(nonce));
-	
+
 	cbs.process_dialog_terminated=NULL;
 	cbs.process_io_error=NULL;
 	cbs.process_request_event=(void (*)(void *user_ctx, const belle_sip_request_event_t *event))register_process_request_event;
@@ -970,6 +1104,232 @@ static void test_register_with_next_nonce(void) {
 	belle_sip_provider_remove_listening_point(prov, server_lp);
 }
 
+static void test_channel_load_process_response_event(void* user_ctx, const belle_sip_response_event_t *event){
+
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_response_event_get_response(event))) {
+		return;
+	}
+	belle_sip_message("process_response_event [%i] [%s] number [%i]"
+					  ,belle_sip_response_get_status_code(belle_sip_response_event_get_response(event))
+					  ,belle_sip_response_get_reason_phrase(belle_sip_response_event_get_response(event))
+						,*(int*)user_ctx);
+
+	(*(int*)user_ctx) ++;
+
+	return;
+}
+
+#define NUMBER_OF_TRANS 20
+static void test_channel_load(void){
+	belle_sip_listening_point_t *lp=belle_sip_provider_get_listening_point(prov,"TCP");
+	BC_ASSERT_PTR_NOT_NULL(lp);
+
+	if (lp) {
+		belle_sip_request_t *req;
+		belle_sip_client_transaction_t *tr[NUMBER_OF_TRANS];
+		char identity[128];
+		char uri[128];
+		char tmp[4];
+		snprintf(identity,sizeof(identity),"Tester <sip:bellesip%s@%s>",belle_sip_random_token(tmp, sizeof(tmp)),auth_domain);
+		snprintf(uri,sizeof(uri),"sip:%s;transport=tcp",auth_domain);
+		belle_sip_listener_callbacks_t listener_callbacks;
+		listener_callbacks.process_dialog_terminated = NULL;
+		listener_callbacks.process_io_error = NULL;
+		listener_callbacks.process_request_event = NULL;
+		listener_callbacks.process_response_event = test_channel_load_process_response_event;
+		listener_callbacks.process_timeout = NULL;
+		listener_callbacks.process_transaction_terminated = NULL;
+		listener_callbacks.process_auth_requested = NULL;
+		listener_callbacks.listener_destroyed = NULL;
+
+		int number_of_response=0;
+		int i;
+		belle_sip_listener_t *listener = belle_sip_listener_create_from_callbacks(&listener_callbacks, (void *)&number_of_response);
+		belle_sip_provider_add_sip_listener(prov,listener);
+
+		for (i = 0; i <NUMBER_OF_TRANS; i++ ) {
+			req = belle_sip_request_create(
+										   belle_sip_uri_parse(uri),
+										   "REGISTER",
+										   belle_sip_provider_create_call_id(prov),
+										   belle_sip_header_cseq_create(20,"REGISTER"),
+										   belle_sip_header_from_create2(identity,BELLE_SIP_RANDOM_TAG),
+										   belle_sip_header_to_create2(identity,NULL),
+										   belle_sip_header_via_new(),
+										   70);
+			tr[i] = belle_sip_provider_create_client_transaction(prov, req);
+			belle_sip_object_ref(tr[i]);
+
+		}
+		belle_sip_client_transaction_send_request(tr[0]);
+		for (int j=0; j < 10 && number_of_response < 1; j++) {
+			belle_sip_stack_sleep(stack, 500);
+		}
+		belle_sip_channel_t *chan = belle_sip_provider_get_channel(prov,tr[0]->next_hop);
+		belle_sip_socket_t sock = belle_sip_source_get_socket((belle_sip_source_t*)chan);
+		size_t sendbuff = 50;
+		int err = bctbx_setsockopt((bctbx_socket_t)sock, SOL_SOCKET, SO_SNDBUF, &sendbuff, sizeof(sendbuff));
+		if (err != 0){
+			belle_sip_error("bctbx_setsockopt SOL_SOCKET failed: [%s]",belle_sip_get_socket_error_string());
+		}
+
+		for (i = 1; i <NUMBER_OF_TRANS; i++ ) {
+			belle_sip_client_transaction_send_request(tr[i]);
+			belle_sip_object_ref(tr[i]);
+		}
+
+		for (int j=0; j < 1000 && number_of_response < NUMBER_OF_TRANS; j++) {
+			belle_sip_stack_sleep(stack, 50);
+		}
+		BC_ASSERT_EQUAL(number_of_response,NUMBER_OF_TRANS,int,"%d");
+		belle_sip_provider_remove_sip_listener(prov,listener);
+	}
+}
+
+static void udp_socket(int  port_mode) {
+	belle_sip_listening_point_t *old_lp=belle_sip_provider_get_listening_point(prov,"UDP");
+	belle_sip_object_ref(old_lp);
+	belle_sip_listening_point_t *new_lp=belle_sip_stack_create_listening_point(stack, "0.0.0.0", port_mode, "UDP");
+	BC_ASSERT_PTR_NOT_NULL(old_lp);
+	BC_ASSERT_PTR_NOT_NULL(new_lp);
+	int number_of_response=0;
+
+	if (old_lp && new_lp) {
+		belle_sip_request_t *req1, *req2;
+		belle_sip_client_transaction_t *tr1, *tr2, *tr3, *tr4;
+		char identity[128];
+		char uri[128];
+		
+		belle_sip_provider_remove_listening_point(prov, old_lp);
+		belle_sip_provider_add_listening_point(prov, new_lp);
+		
+		belle_sip_listener_callbacks_t listener_callbacks;
+		listener_callbacks.process_dialog_terminated = NULL;
+		listener_callbacks.process_io_error = NULL;
+		listener_callbacks.process_request_event = NULL;
+		listener_callbacks.process_response_event = test_channel_load_process_response_event;
+		listener_callbacks.process_timeout = NULL;
+		listener_callbacks.process_transaction_terminated = NULL;
+		listener_callbacks.process_auth_requested = NULL;
+		listener_callbacks.listener_destroyed = NULL;
+
+		belle_sip_listener_t *listener = belle_sip_listener_create_from_callbacks(&listener_callbacks, (void *)&number_of_response);
+		belle_sip_provider_add_sip_listener(prov,listener);
+		char tmp[4];
+		snprintf(identity,sizeof(identity),"Tester <sip:bellesip%s@%s>",belle_sip_random_token(tmp, sizeof(tmp)),test_domain);
+		snprintf(uri,sizeof(uri),"sip:%s",auth_domain);
+		req1 = belle_sip_request_create(
+										belle_sip_uri_parse(uri),
+										"REGISTER",
+										belle_sip_provider_create_call_id(prov),
+										belle_sip_header_cseq_create(20,"REGISTER"),
+										belle_sip_header_from_create2(identity,BELLE_SIP_RANDOM_TAG),
+										belle_sip_header_to_create2(identity,NULL),
+										belle_sip_header_via_new(),
+										70);
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req1),BELLE_SIP_HEADER(belle_sip_header_expires_create(60)));
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req1),BELLE_SIP_HEADER(belle_sip_header_contact_new()));
+
+		tr1 = belle_sip_provider_create_client_transaction(prov, req1);
+		belle_sip_object_ref(tr1);
+		belle_sip_client_transaction_send_request(tr1);
+		
+		snprintf(uri,sizeof(uri),"sip:%s",test_domain);
+		req2 = belle_sip_request_create(
+										belle_sip_uri_parse(uri),
+										"REGISTER",
+										belle_sip_provider_create_call_id(prov),
+										belle_sip_header_cseq_create(20,"REGISTER"),
+										belle_sip_header_from_create2(identity,BELLE_SIP_RANDOM_TAG),
+										belle_sip_header_to_create2(identity,NULL),
+										belle_sip_header_via_new(),
+										70);
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req2),BELLE_SIP_HEADER(belle_sip_header_expires_create(60)));
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req2),BELLE_SIP_HEADER(belle_sip_header_contact_new()));
+
+		tr2 = belle_sip_provider_create_client_transaction(prov, req2);
+		belle_sip_object_ref(tr2);
+		belle_sip_client_transaction_send_request(tr2);
+
+	
+		for (int j=0; j < 10 && number_of_response < 1; j++) {
+			belle_sip_stack_sleep(stack, 500);
+		}
+		belle_sip_channel_t *chan1 = belle_sip_provider_get_channel(prov,tr1->next_hop);
+		belle_sip_object_ref(chan1);
+		belle_sip_channel_t *chan2 = belle_sip_provider_get_channel(prov,tr2->next_hop);
+		belle_sip_object_ref(chan2);
+		BC_ASSERT_PTR_NOT_EQUAL(chan1, chan2);
+		belle_sip_socket_t sock1 = belle_sip_source_get_socket((belle_sip_source_t*)chan1);
+		belle_sip_socket_t sock2 = belle_sip_source_get_socket((belle_sip_source_t*)chan2);
+		if (port_mode == BELLE_SIP_LISTENING_POINT_DONT_BIND) {
+			BC_ASSERT_NOT_EQUAL((int)sock1, (int)sock2,int,"%d");
+			BC_ASSERT_EQUAL(belle_sip_listening_point_get_port(new_lp),BELLE_SIP_LISTENING_POINT_DONT_BIND, int, "%d");
+			int chan1_local_port, chan2_local_port;
+			char socket_local_port[8];
+			struct sockaddr_storage laddr;
+			memset(&laddr, 0, sizeof(laddr));
+			socklen_t lslen=sizeof(laddr);
+			bctbx_getsockname(sock1,(struct sockaddr*)&laddr,&lslen);
+			bctbx_getnameinfo((struct sockaddr*)&laddr,lslen,NULL,0,socket_local_port,sizeof(socket_local_port),NI_NUMERICSERV);
+			BC_ASSERT_PTR_NOT_NULL(belle_sip_channel_get_local_address(chan1, &chan1_local_port));
+			BC_ASSERT_EQUAL(chan1_local_port, atoi(socket_local_port), int, "%d");
+			BC_ASSERT_PTR_NOT_NULL(belle_sip_channel_get_local_address(chan2, &chan2_local_port));
+			BC_ASSERT_NOT_EQUAL(chan1_local_port, chan2_local_port, int, "%d");
+		} else if (port_mode > 0 || port_mode == BELLE_SIP_LISTENING_POINT_RANDOM_PORT ) {
+			BC_ASSERT_EQUAL((int)sock1, (int)sock2,int,"%d");
+		} else
+			BC_FAIL("Unsupported port mode");
+		
+		BC_ASSERT_EQUAL(number_of_response,2,int,"%d");
+		
+		belle_sip_stack_set_tx_delay(stack,500);
+		tr3 = belle_sip_provider_create_client_transaction(prov, req1);
+		belle_sip_object_ref(tr3);
+		belle_sip_client_transaction_send_request(tr3);
+		tr4 = belle_sip_provider_create_client_transaction(prov, req2);
+		belle_sip_object_ref(tr4);
+		belle_sip_client_transaction_send_request(tr4);
+		belle_sip_channel_t *chan3 = belle_sip_provider_get_channel(prov,tr3->next_hop);
+		belle_sip_channel_t *chan4 = belle_sip_provider_get_channel(prov,tr4->next_hop);
+
+		BC_ASSERT_PTR_EQUAL(chan1, chan3);
+		BC_ASSERT_PTR_EQUAL(chan2, chan4);
+
+		BC_ASSERT_EQUAL((int)sock1, (int)belle_sip_source_get_socket((belle_sip_source_t*)chan3),int,"%d");
+		BC_ASSERT_EQUAL((int)sock2, (int)belle_sip_source_get_socket((belle_sip_source_t*)chan4),int,"%d");
+
+		bctbx_socket_close(sock1);
+		
+		belle_sip_stack_sleep(stack, 2000);
+		
+		if (port_mode == BELLE_SIP_LISTENING_POINT_DONT_BIND) {
+			//only channel 1 is affected
+			BC_ASSERT_EQUAL(belle_sip_channel_get_state(chan1), BELLE_SIP_CHANNEL_ERROR, belle_sip_channel_state_t, "%d");
+			BC_ASSERT_EQUAL(belle_sip_channel_get_state(chan2), BELLE_SIP_CHANNEL_READY, belle_sip_channel_state_t, "%d");
+			BC_ASSERT_EQUAL(number_of_response,2+1,int,"%d");
+		} else if (port_mode > 0 || port_mode == BELLE_SIP_LISTENING_POINT_RANDOM_PORT ) {
+			BC_ASSERT_EQUAL(belle_sip_channel_get_state(chan1), BELLE_SIP_CHANNEL_ERROR, belle_sip_channel_state_t, "%d");
+			BC_ASSERT_EQUAL(belle_sip_channel_get_state(chan2), BELLE_SIP_CHANNEL_ERROR, belle_sip_channel_state_t, "%d");
+			BC_ASSERT_EQUAL(number_of_response,2,int,"%d");
+		} else
+			BC_FAIL("Unsupported port mode");
+
+		belle_sip_object_unref(chan1);
+		belle_sip_object_unref(chan2);
+		belle_sip_provider_remove_sip_listener(prov,listener);
+		belle_sip_provider_remove_listening_point(prov, new_lp);
+		belle_sip_provider_add_listening_point(prov, old_lp);
+	}
+}
+static void udp_single_socket(void) {
+	udp_socket(BELLE_SIP_LISTENING_POINT_RANDOM_PORT);
+}
+
+static void udp_multiple_socket(void) {
+	udp_socket(BELLE_SIP_LISTENING_POINT_DONT_BIND);
+}
+		
 test_t register_tests[] = {
 	TEST_NO_TAG("Stateful UDP", stateful_register_udp),
 	TEST_NO_TAG("Stateful UDP with keep-alive", stateful_register_udp_with_keep_alive),
@@ -987,6 +1347,7 @@ test_t register_tests[] = {
 	TEST_NO_TAG("Bad TCP request", test_bad_request),
 	TEST_NO_TAG("Authenticate", test_register_authenticate),
 	TEST_NO_TAG("TLS client cert authentication", test_register_client_authenticated),
+	TEST_NO_TAG("TLS client cert bad ciphersuites", test_register_client_bad_ciphersuites),
 	TEST_NO_TAG("Channel inactive", test_register_channel_inactive),
 	TEST_NO_TAG("Channel moving to error test and cleaned", test_channel_moving_to_error_and_cleaned),
 	TEST_NO_TAG("TCP connection failure", test_connection_failure),
@@ -998,7 +1359,10 @@ test_t register_tests[] = {
 	TEST_NO_TAG("Register with DNS SRV failover TLS with http proxy", register_dns_srv_tls_with_http_proxy),
 	TEST_NO_TAG("Register with DNS load-balancing", register_dns_load_balancing),
 	TEST_NO_TAG("Nonce reutilization", reuse_nonce),
-	TEST_NO_TAG("Next Nonce", test_register_with_next_nonce)
+	TEST_NO_TAG("Next Nonce", test_register_with_next_nonce),
+	TEST_NO_TAG("Channel load", test_channel_load),
+	TEST_NO_TAG("UDP multiple channels single socket", udp_single_socket),
+	TEST_NO_TAG("UDP multiple channels multiple sockets", udp_multiple_socket)
 };
 
 test_suite_t register_test_suite = {"Register", register_before_all, register_after_all, NULL,
